      PROGRAM NETPRT
C----------------------------------------------------------------------*
C     PROGRAM:      NETPRT                                             *
C----------------------------------------------------------------------*
C     LANGUAGE:     VAX-11 FORTRAN V2.4                                *
C     SYSTEM:       VAX-11/780 VAX/VMS V2.4                            *
C                   MOSTEK CORPORATION                                 *
C                   COMPUTER AIDS TO DESIGN DEPARTMENT                 *
C                   1215 WEST CROSBY ROAD                              *
C                   CARROLLTON, TEXAS  75006                           *
C                   (214)  323-8813                                    *
C----------------------------------------------------------------------*
C     PROGRAMMER:   KEVIN KLUGHART                                     *
C----------------------------------------------------------------------*
C     DATE:         04-26-82 @ 12:00 CDST                              *
C----------------------------------------------------------------------*
C     PURPOSE:      THIS PROGRAM IS DESIGNED AS A NON-PRIVILEGED       *
C                   UTILITY TO ENABLE ORDINARY USERS TO QUEUE FILES    *
C                   FOR PRINTING FROM ONE NODE ON THE DECNET NETWORK   *
C                   TO ANY OTHER NODE.  THIS PROGRAM REQUIRES THE      *
C                   NETMBX AND TMPMBX PRIVILEGES FOR PROPER OPERATION. *
C----------------------------------------------------------------------*
C     NOTES:        DEFINE THE FOLLOWING SYMBOL IN THE SYSTEM USER     *
C                   LOGIN COMMAND PROCEDURE:                           *
C                                                                      *
C                           $ NETPRT :== $NETPRT                       *
C                                                                      *
C                   THIS WILL ENABLE USER SYNTAX OF THE FORM:          *
C                                                                      *
C                           $ NETPRT node filespec [print options]     *
C                                                                      *
C                   WHERE node IS THE REMOTE NODE TO WHICH THE FILE    *
C                   IS TO BE PRINTED, filespec IS THE FILE             *
C                   SPECIFICATION OF THE FILE[S] TO BE PRINTED, AND    *
C                   print options ARE THE OPTIONAL QUALIFIERS TO BE    *
C                   ASSOCIATED WITH THE 'PRINT' OR 'SPOOL' COMMAND     *
C                   ON THE REMOTE NODE.  FOR EXAMPLE:                  *
C                                                                      *
C                           $NETPRT CADEGR TEST.LIS /NOFEED            *
C                                                                      *
C                   THE ABOVE COMMAND LINE PRINTS FILE TEST.LIS WHICH  *
C                   RESIDES AT NODE SYS$NODE ON CADEGR'S PRINTER WITH  *
C                   A 'NOFEED' PRINT OPTION.                           *
C----------------------------------------------------------------------*
C     EXTERNALS:    STRING LIBRARY                                     *
C----------------------------------------------------------------------*
C     PARAMETERS                                                       *
C----------------------------------------------------------------------*
      PARAMETER BATCH_PROC       = 'NETPRT.COM'   ! REMOTE BATCH FILE
      PARAMETER BATCH_UNIT       = 1              ! REMOTE BATCH FILE UNIT
      PARAMETER BLANK            = ' '            ! DEFINE SPACE CHARACTER
      PARAMETER DEFOPT           = '/DELETE'      ! DEFAULT PRINT OPTIONS
      PARAMETER DELIMITER        = '/'            ! DELIMIT PRINT OPTIONS
      PARAMETER FIVE             = 5              ! OFFSET FOR 3 BYTE NODE ALIAS
      PARAMETER FULL             = 'F'X           ! FULL ERROR REPORT BIT MASK
      PARAMETER LOCAL_NODE       = 'SYS$NODE'     ! DEFINED IF DECNET RUNNING
      PARAMETER MAX              = 255            ! MAXIMUM INPUT RECORD
      PARAMETER NETNOD           = '::'           ! INDICATES NETWORK NODE
      PARAMETER ONE              = 1              ! USEFUL CONSTANT
      PARAMETER RIGHT            = ']'            ! RIGHT BRACKET (DIRECTORY)
      PARAMETER SEMICOLON        = ';'            ! SEMICOLON (VERSION)
      PARAMETER TWO              = 2              ! USEFUL CONSTANT
      PARAMETER ZERO             = 0              ! USEFUL CONSTANT
C----------------------------------------------------------------------*
C     RMS STATUS CODES                                                 *
C----------------------------------------------------------------------*
      EXTERNAL RMS$_FNF
      EXTERNAL RMS$_NMF
      EXTERNAL RMS$_NORMAL
C----------------------------------------------------------------------*
C     SYSTEM SERVICES STATUS CODES                                     *
C----------------------------------------------------------------------*
      EXTERNAL SS$_NORMAL
C----------------------------------------------------------------------*
C     CHARACTER                                                        *
C----------------------------------------------------------------------*
      CHARACTER*(MAX) BATCH_FILE
      CHARACTER*(MAX) ERRMSG
      CHARACTER*(MAX) FILE
      CHARACTER*(MAX) LINE
      CHARACTER*(MAX) LOCAL
      CHARACTER*(MAX) NODE
      CHARACTER*(MAX) OPTIONS
C----------------------------------------------------------------------*
C     INTEGER*4                                                        *
C----------------------------------------------------------------------*
      INTEGER*4 BATLEN
      INTEGER*4 ERRLEN
      INTEGER*4 FILLEN
      INTEGER*4 LENGTH
      INTEGER*4 LIB$DO_COMMAND
      INTEGER*4 LIB$GET_FOREIGN
      INTEGER*4 LIB$PUT_OUTPUT
      INTEGER*4 LIMIT
      INTEGER*4 LOCLEN
      INTEGER*4 NODLEN
      INTEGER*4 OPTLEN
      INTEGER*4 POINTER
      INTEGER*4 RMS$_FNF
      INTEGER*4 RMS$_NMF
      INTEGER*4 RMS$_NORMAL
      INTEGER*4 RMS$PARSE
      INTEGER*4 RMS$SEARCH
      INTEGER*4 SS$_NORMAL      
      INTEGER*4 STATUS
      INTEGER*4 SYS$TRNLOG

C----------------------------------------------------------------------*
C     SECTION 000                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     INITIALIZATION                                                   *
C                                                                      *
C----------------------------------------------------------------------*
C     DETERMINE LOCAL NODE NAME AND ABORT IF NETWORK IS DOWN           *
C----------------------------------------------------------------------*
      STATUS=SYS$TRNLOG(LOCAL_NODE,LOCLEN,LOCAL,,,)
      IF (STATUS .EQ. %LOC(SS$_NORMAL)) GO TO 100

      CALL LIB$PUT_OUTPUT('***  NETWORK  NOT CURRENTLY AVAILABLE  ***')
      GO TO 800

C----------------------------------------------------------------------*
C     SECTION 100                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     REQUEST COMMAND LINE OF FORM:  $NETPRT NODE FILE OPTIONS         *
C                                                                      *
C----------------------------------------------------------------------*
C     GET NODE NAME AND INCLUDE NODE IDENTIFICATION ('::') IF NEEDED   *
C----------------------------------------------------------------------*
  100 CONTINUE
      STATUS=LIB$GET_FOREIGN(LINE,'  Enter Node Name: ',NODLEN)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 800
      IF (NODLEN .EQ. ZERO) GO TO 100

      CALL GET_WORD(LINE,NODE)
      IF (INDEX(NODE,NETNOD) .NE. ZERO) GO TO 110
      NODLEN=LENGTH(NODE)
      NODE((NODLEN+ONE):)=NETNOD
      NODLEN=NODLEN+TWO
C----------------------------------------------------------------------*
C     GET FILE NAME AND ANY USER OPTIONS                               *
C----------------------------------------------------------------------*
  110 CONTINUE
      FILE=LINE
      FILLEN=LENGTH(FILE)

  120 CONTINUE
      IF (FILLEN .EQ. ZERO) 
     1     STATUS=LIB$GET_FOREIGN(FILE,'  Enter File Name: ',FILLEN)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 800
      IF (FILLEN .EQ. ZERO) GO TO 120
C----------------------------------------------------------------------*
C     SAVE OPTIONS FOR PRINT QUEUE ON REMOTE NODE                      *
C----------------------------------------------------------------------*
      POINTER=INDEX(FILE,DELIMITER)
      OPTIONS=DEFOPT

      IF (POINTER .NE. ZERO) THEN
           OPTIONS=DEFOPT//FILE(POINTER:)
           FILE=FILE(ONE:(POINTER-ONE))
           FILLEN=POINTER-ONE
      ENDIF

      OPTLEN=LENGTH(OPTIONS)

C----------------------------------------------------------------------*
C     SECTION 200                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     TEST FILE SPECIFICATION FOR FILE EXISTANCE AND GET FILE SPEC     *
C                                                                      *
C----------------------------------------------------------------------*
C     PARSE THE FILE SPECIFICATION IN PREPARATION FOR WILDCARD SEARCH  *
C----------------------------------------------------------------------*
  200 CONTINUE
      STATUS=RMS$PARSE(FILE(ONE:FILLEN))
      IF (STATUS .NE. %LOC(RMS$_NORMAL)) GO TO 800
C----------------------------------------------------------------------*
C     SEARCH FOR NEXT FILE SPECIFICATION IN WILDCARD SEARCH            *
C----------------------------------------------------------------------*
  250 CONTINUE
      STATUS=RMS$SEARCH(FILE)
      FILLEN=LENGTH(FILE)
      IF (STATUS .EQ. %LOC(RMS$_NMF)) GO TO 900
      IF (STATUS .NE. %LOC(RMS$_NORMAL)) GO TO 800

      CALL LIB$PUT_OUTPUT(BLANK)
      CALL LIB$PUT_OUTPUT('  File '//
     1     LOCAL(ONE:LOCLEN)//FILE(ONE:FILLEN))

C----------------------------------------------------------------------*
C     SECTION 300                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     DETERMINE WHAT TYPE OF NODE FILE IS TO BE ROUTED TO:  RSX OR VMS *
C                                                                      *
C----------------------------------------------------------------------*
C     SELECT MCR OR DCL PROCEDURES BASED ON REMOTE NODE TYPE           *
C----------------------------------------------------------------------*
  300 CONTINUE
      IF (NODE(ONE:NODLEN) .EQ. 'CADRJE::') GO TO 500
      IF (NODE(ONE:NODLEN) .EQ. 'CADCYB::') GO TO 500

C----------------------------------------------------------------------*
C     SECTION 400                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     OPEN BATCH COMMAND FILE ON REMOTE NODE (VMS/DCL)                 *
C                                                                      *
C----------------------------------------------------------------------*
C     CONVERT LOCAL FILENAME TO REMOTE NODE FILENAME:                  *
C          ELIMINATE DEVICE AND DIRECTORY INFORMATION                  *
C----------------------------------------------------------------------*
  400 CONTINUE
      POINTER=INDEX(FILE,RIGHT)
      OPEN (UNIT=BATCH_UNIT,
     1      NAME=NODE(ONE:NODLEN)//BATCH_PROC,
     2      CARRIAGECONTROL='LIST',TYPE='NEW',ERR=850)
      
      INQUIRE (UNIT=BATCH_UNIT,NAME=BATCH_FILE)
      CALL DELETE_CONTROL(BATCH_FILE)
      BATLEN=LENGTH(BATCH_FILE)
C----------------------------------------------------------------------*
C     BUILD REMOTE BATCH COMMAND PROCEDURE                             *
C----------------------------------------------------------------------*
      WRITE (BATCH_UNIT,4000) '$ ON ERROR THEN CONTINUE'

      WRITE (BATCH_UNIT,4000) '$ COPY '//
     1     LOCAL(ONE:LOCLEN)//FILE(ONE:FILLEN)//
     2     ' '//FILE((POINTER+ONE):FILLEN)

      WRITE (BATCH_UNIT,4000) '$ PRINT '//
     1     FILE((POINTER+ONE):FILLEN)//OPTIONS(ONE:OPTLEN)
C----------------------------------------------------------------------*
C     CLOSE REMOTE BATCH COMMAND FILE AND INFORM USER OF TRANSFER      *
C----------------------------------------------------------------------*
      CLOSE (UNIT=BATCH_UNIT,DISPOSE='SUBMIT/DELETE')

      CALL LIB$PUT_OUTPUT('       Successfully queued to remote node '//
     1     NODE(ONE:NODLEN))

      GO TO 900

C----------------------------------------------------------------------*
C     SECTION 500                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     OPEN BATCH COMMAND FILE ON REMOTE NODE (RSX/MCR)                 *
C                                                                      *
C----------------------------------------------------------------------*
C     CONVERT LOCAL FILENAME TO REMOTE FILENAME:                       *
C----------------------------------------------------------------------*
  500 CONTINUE
      OPEN (UNIT=BATCH_UNIT,
     1      NAME=NODE(ONE:NODLEN)//BATCH_PROC,
     2      CARRIAGECONTROL='LIST',TYPE='NEW',ERR=850)

      INQUIRE (UNIT=BATCH_UNIT,NAME=BATCH_FILE)
      CALL DELETE_CONTROL(BATCH_FILE)
      BATLEN=LENGTH(BATCH_FILE)
C----------------------------------------------------------------------*
C     BUILD REMOTE BATCH COMMAND PROCEDURE                             *
C----------------------------------------------------------------------*
      WRITE (BATCH_UNIT,4000) '.ENABLE QUIET'
      WRITE (BATCH_UNIT,4000) 'SET /UIC=[200,200]'
      WRITE (BATCH_UNIT,4000) 'NFT LB:NETPRT.LIS=XRB'//
     1     LOCAL(FIVE:LOCLEN)//FILE(TWO:FILLEN)
      WRITE (BATCH_UNIT,4000) 'PRI LB:NETPRT.LIS /DELETE'
C----------------------------------------------------------------------*
C     CLOSE REMOTE BATCH COMMAND FILE AND INFORM USER OF TRANSFER      *
C----------------------------------------------------------------------*
      CLOSE (UNIT=BATCH_UNIT)
      CALL LIB$PUT_OUTPUT('       Successfully queued to remote node '//
     1     NODE(ONE:NODLEN))

      STATUS=LIB$DO_COMMAND('$SUBMIT '//BATCH_FILE(ONE:BATLEN)//
     1     ' /REMOTE')
      GO TO 900

C----------------------------------------------------------------------*
C     SECTION 800                                                      *
C----------------------------------------------------------------------*
C     ERROR TRAP:     DISPLAY SYSTEM ERROR MESSAGE AND ABORT EXECUTION *
C----------------------------------------------------------------------*
  800 CONTINUE
      CALL LIB$PUT_OUTPUT(BLANK)
      CALL LIB$PUT_OUTPUT('  ***  FATAL ERROR ENCOUNTERED  ***')

  810 CONTINUE
      CALL SYS$GETMSG(%VAL(STATUS),MSGLEN,ERRMSG,%VAL(FULL),)
      CALL LIB$PUT_OUTPUT(BLANK//BLANK//ERRMSG(ONE:MSGLEN))

      GO TO 900
C----------------------------------------------------------------------*
C     ERROR TRAP:     PREPROCESS FILE ACCESS ERRORS                    *
C----------------------------------------------------------------------*
  850 CONTINUE

      CALL LIB$PUT_OUTPUT(BLANK)
      CALL LIB$PUT_OUTPUT('  ***  FATAL FILE ERROR ENCOUNTERED  ***')
      CALL ERRSNS(,STATUS,,,)

      GO TO 810

C----------------------------------------------------------------------*
C     SECTION 900                                                      *
C----------------------------------------------------------------------*
C     PROGRAM TERMINATION                                              *
C----------------------------------------------------------------------*
  900 CONTINUE
C----------------------------------------------------------------------*
C     FORMATS                                                          *
C----------------------------------------------------------------------*
 4000 FORMAT(A)
C----------------------------------------------------------------------*
C     END OF MAIN PROGRAM NETPRT                                       *
C----------------------------------------------------------------------*
      END
