      PROGRAM SPOOLER
C----------------------------------------------------------------------*
C     PROGRAM:      SPOOLER                                            *
C----------------------------------------------------------------------*
C     LANGUAGE:     VAX-11 FORTRAN IV PLUS 01                          *
C     SYSTEM:       VAX-11/780                                         *
C                   MOSTEK CORPORATION                                 *
C                   COMPUTER AIDS TO DESIGN DEPARTMENT                 *
C                   1215 WEST CROSBY ROAD                              *
C                   CARROLLTON, TEXAS  75006                           *
C                   (214)  323-8813                                    *
C----------------------------------------------------------------------*
C     PROGRAMMER:   KEVIN KLUGHART                                     *
C----------------------------------------------------------------------*
C     DATE:         04-15-82 @ 06:00 CDST                              *
C----------------------------------------------------------------------*
C     PURPOSE:      THIS PROGRAM IS THE MAIN REMOTE PRINT SPOOLER      *
C                   FOR THE RJE/HASP PDP-11/34 COMPUTER SYSTEM.        *
C                   THE MAIN FUNCTION OF THIS PROGRAM IS TO TRANSFER   *
C                   LISTING FILES FROM THE LOCAL VAX NODE OVER DECNET  *
C                   TO THE PDP-11/34 (HASP) SYSTEM LINEPRINTER, WHICH  *
C                   IS RESIDENT IN A REMOTE USER LOCATION AT MOSTEK.   *
C----------------------------------------------------------------------*
C     NOTES:        THIS MAIN PROGRAM IS MEANT TO BE USED IN UNISON    *
C                   WITH THE 'SPLCOMAND.FOR' PROGRAM, WHICH VALIDATES  *
C                   THE USER FILE TO BE PRINTED AND SUBMITS IT VIA     *
C                   A MAILBOX COMMAND TO THIS PROGRAM WHICH THEN       *
C                   PROCESSES IT AND SPOOLS IT TO THE REMOTE PRINTER.  *
C----------------------------------------------------------------------*
C     EXTERNALS:    STRING LIBRARY                                     *
C----------------------------------------------------------------------*
C     PARAMETERS                                                       *
C----------------------------------------------------------------------*
      PARAMETER BLANK            = ' '            ! DEFINE SPACE CHARACTER
      PARAMETER FILETYPE         = '.LST'         ! OUTPUT FILE TYPE
      PARAMETER FORM_FEED_CODE   = '0C'X          ! ASCII FORM FEED CODE
      PARAMETER MAXLBL           = 128            ! BANNER LENGTH (LINES)
      PARAMETER MAXLIN           = 132            ! MAXIMUM PRINTER WIDTH
      PARAMETER MAXVER           = 5              ! MAXIMUM VERSION LENGTH
      PARAMETER MAXWID           = 255            ! MAXIMUM INPUT RECORD
      PARAMETER NEW_BAR_CODE     = '21'X          ! ASCII CODE FOR '!'
      PARAMETER ONE              = 1              ! USEFUL CONSTANT
      PARAMETER PERIOD           = '.'            ! ASCII PERIOD
      PARAMETER PERMANENT        = 1              ! PERMANENT MAILBOX FLAG
      PARAMETER PRINTER_DELAY    = 10000          ! SPOOL DELAY IN MILLISECONDS
      PARAMETER PRINTER_MAILBOX  = 'HASP_PRT_MBX' ! REMOTE PRINTER MAILBOX
C----------------------------------------------------------------------*
C     THE FOLLOWING PARAMETER DEFINES THE REMOTE PRINTER DEVICE        *
C----------------------------------------------------------------------*
      PARAMETER PRINTER_NAME     = 'CADCYB::LP:'  ! PRINTER NAME
      PARAMETER PRINTER_UNIT     = 2              ! PRINTER LISTING UNIT
      PARAMETER READER           = 1              ! INPUT FILE UNIT
      PARAMETER RIGHT            = ']'            ! RIGHT BRACKET (DIRECTORY)
      PARAMETER SEMICOLON        = ';'            ! SEMICOLON (VERSION)
      PARAMETER TWO              = 2              ! USEFUL CONSTANT
      PARAMETER VERT_BAR_CODE    = '7C'X          ! ASCII VERTICAL BAR CODE
      PARAMETER ZERO             = 0              ! USEFUL CONSTANT
C----------------------------------------------------------------------*
C     EXTERNAL REFERENCES                                              *
C----------------------------------------------------------------------*
C     INPUT/OUTPUT FUNCTION CODES                                      *
C----------------------------------------------------------------------*
      EXTERNAL IO$_READVBLK
C----------------------------------------------------------------------*
C     RMS STATUS CODES                                                 *
C----------------------------------------------------------------------*
      EXTERNAL RMS$_FNF
      EXTERNAL RMS$_NMF
      EXTERNAL RMS$_NORMAL
C----------------------------------------------------------------------*
C     SYSTEM SERVICES STATUS CODES                                     *
C----------------------------------------------------------------------*
      EXTERNAL SS$_NORMAL
      EXTERNAL SS$_SUPERSEDE
C----------------------------------------------------------------------*
C     CHARACTER                                                        *
C----------------------------------------------------------------------*
      CHARACTER*(MAXWID) CONTROL
      CHARACTER*(MAXWID) MAILBOX
      CHARACTER*(MAXWID) PRINTER_FILE
      CHARACTER*(MAXWID) PRINTER_LINE
      CHARACTER*(MAXLIN) STRING
C----------------------------------------------------------------------*
C     INTEGER*4                                                        *
C----------------------------------------------------------------------*
      INTEGER*4 LENGTH
      INTEGER*4 LIMIT
      INTEGER*4 POINTER
      INTEGER*4 PRINTER_CHANNEL
      INTEGER*4 RMS$PARSE
      INTEGER*4 RMS$SEARCH
      INTEGER*4 STATUS
      INTEGER*4 SYS$CREMBX
      INTEGER*4 SYS$DELMBX
      INTEGER*4 SYS$QIOW
      INTEGER*4 VERSION
C----------------------------------------------------------------------*
C     LOGICAL*4                                                        *
C----------------------------------------------------------------------*
      LOGICAL*4 ERROR
      LOGICAL*4 NODATA
C----------------------------------------------------------------------*
C     SECTION 100                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     INITIALIZE THE DATA INPUT MAILBOX (SPOOLED VAX INPUT)            *
C                                                                      *
C----------------------------------------------------------------------*
  100 CONTINUE
      STATUS=SYS$CREMBX(%VAL(PERMANENT),PRINTER_CHANNEL,%VAL(MAXWID)
     1     ,,,,PRINTER_MAILBOX)

      IF ((STATUS .NE. %LOC(SS$_NORMAL)) .AND.
     1     (STATUS .NE. %LOC(SS$_SUPERSEDE))) GO TO 800
C----------------------------------------------------------------------*
C     SECTION 200                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     READ MAILBOX AND ACTIVATE TRANSFER IF DATA IS PRESENT            *
C                                                                      *
C----------------------------------------------------------------------*
  200 CONTINUE
      MAILBOX=BLANK

      STATUS=SYS$QIOW(,%VAL(PRINTER_CHANNEL),%VAL(%LOC(IO$_READVBLK))
     1     ,,,,%REF(MAILBOX),%VAL(MAXWID),,,,)

      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 800
C----------------------------------------------------------------------*
C     PARSE THE FILE SPECIFICATION IN PREPARATION FOR WILDCARD SEARCH  *
C----------------------------------------------------------------------*
      CALL DELETE_CONTROL(MAILBOX)
      LIMIT=LENGTH(MAILBOX)
      IF (LIMIT .EQ. ZERO) GO TO 200

      STATUS=RMS$PARSE(MAILBOX(ONE:LIMIT))
      IF (STATUS .NE. %LOC(RMS$_NORMAL)) GO TO 800
C----------------------------------------------------------------------*
C     SECTION 300                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     OPEN FILE TO BE TRANSFERED TO REMOTE PRINTER                     *
C                                                                      *
C----------------------------------------------------------------------*
C     SEARCH FOR NEXT FILE SPECIFICATION IN WILDCARD SEARCH            *
C----------------------------------------------------------------------*
  300 CONTINUE
      STATUS=RMS$SEARCH(PRINTER_FILE)
      IF (STATUS .EQ. %LOC(RMS$_FNF)) GO TO 200
      IF (STATUS .EQ. %LOC(RMS$_NMF)) GO TO 200
      LIMIT=LENGTH(PRINTER_FILE)
      
      OPEN (UNIT=READER,NAME=PRINTER_FILE(ONE:LIMIT),TYPE='OLD',
     1     RECORDSIZE=MAXWID,READONLY,SHARED,ERR=300)
      INQUIRE (UNIT=READER,CARRIAGECONTROL=CONTROL)
C----------------------------------------------------------------------*
C     SECTION 400                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     OPEN SPOOL FILE ON REMOTE NODE                                   *
C                                                                      *
C----------------------------------------------------------------------*
C     INITIALIZE FIRST LINE OF OUTPUT LISTING                          *
C----------------------------------------------------------------------*
  400 CONTINUE
      PRINTER_LINE=PRINTER_FILE(ONE:LIMIT)
C----------------------------------------------------------------------*
C                                                                      *
C     ATTEMPT TO OPEN SPOOL FILE ON REMOTE NODE                        *
C                                                                      *
C----------------------------------------------------------------------*
  410 CONTINUE
      OPEN (UNIT=PRINTER_UNIT,
     1      NAME=PRINTER_NAME,TYPE='NEW',
     2      RECORDSIZE=MAXLIN,CARRIAGECONTROL=CONTROL,ERR=420)
      GO TO 430
C----------------------------------------------------------------------*
C     REMOTE FILE OPEN ERROR - DELAY AND TRY AGAIN LATER               *
C----------------------------------------------------------------------*
  420 CONTINUE
      CALL DELAY(PRINTER_DELAY)
      GO TO 410

  430 CONTINUE
      WRITE (PRINTER_UNIT,4000,ERR=850) PRINTER_LINE(ONE:LIMIT)
C----------------------------------------------------------------------*
C     SECTION 500                                                      *
C----------------------------------------------------------------------*
C     COPY THE LISTING FILE TO THE REMOTE PRINTER                      *
C----------------------------------------------------------------------*
  500 CONTINUE

      READ (READER,4000,END=900) PRINTER_LINE
C----------------------------------------------------------------------*
C     CONVERT SPECIAL CHARACTERS                                       *
C----------------------------------------------------------------------*
  510 CONTINUE
      LIMIT=INDEX(PRINTER_LINE,CHAR(VERT_BAR_CODE))
      IF (LIMIT .EQ. ZERO) GO TO 520
      PRINTER_LINE(LIMIT:LIMIT)=CHAR(NEW_BAR_CODE)
      GO TO 510
C----------------------------------------------------------------------*
C     TRANSFER LINE TO REMOTE SPOOL DEVICE                             *
C----------------------------------------------------------------------*
  520 CONTINUE
      CALL UPPER_CASE(PRINTER_LINE)
      LIMIT=LENGTH(PRINTER_LINE)
      IF (LIMIT .EQ. ZERO) LIMIT=ONE
      IF (LIMIT .GT. MAXLIN) LIMIT=MAXLIN
      WRITE (PRINTER_UNIT,4000,ERR=850) PRINTER_LINE(ONE:LIMIT)
      GO TO 500
C----------------------------------------------------------------------*
C     SECTION 800                                                      *
C----------------------------------------------------------------------*
C     ERROR TRAP:     ABORT EXECUTION                                  *
C----------------------------------------------------------------------*
  800 CONTINUE

      STOP
C----------------------------------------------------------------------*
C     SECTION 850                                                      *
C----------------------------------------------------------------------*
C     PROCESS NETWORK FILE WRITE ERRORS                                *
C----------------------------------------------------------------------*
  850 CONTINUE
      CLOSE (UNIT=PRINTER_UNIT,ERR=410)
      GOTO 410
C----------------------------------------------------------------------*
C     SECTION 900                                                      *
C----------------------------------------------------------------------*
C     PROGRAM TERMINATION:     CLOSE FILES AND PROCESS NEXT FILE       *
C----------------------------------------------------------------------*
  900 CONTINUE
      WRITE (PRINTER_UNIT,5000) CHAR(FORM_FEED_CODE)
      WRITE (PRINTER_UNIT,9000)

      CLOSE (UNIT=READER)
      CLOSE (UNIT=PRINTER_UNIT)

      GO TO 300
C----------------------------------------------------------------------*
C     FORMATS                                                          *
C----------------------------------------------------------------------*
 4000 FORMAT(A)
 5000 FORMAT(1X,A)
 9000 FORMAT(10(1X,128('*')/))
C----------------------------------------------------------------------*
C     END OF MAIN PROGRAM SPOOLER                                      *
C----------------------------------------------------------------------*
      END
