      PROGRAM SPOOL_QUEUE
C----------------------------------------------------------------------*
C     PROGRAM:      SPOOL_QUEUE                                        *
C----------------------------------------------------------------------*
C     LANGUAGE:     VAX-11 FORTRAN IV PLUS 01                          *
C     SYSTEM:       VAX-11/780                                         *
C                   MOSTEK CORPORATION                                 *
C                   COMPUTER AIDS TO DESIGN DEPARTMENT                 *
C                   1215 WEST CROSBY ROAD                              *
C                   CARROLLTON, TEXAS  75006                           *
C                   (214)  323-8813                                    *
C----------------------------------------------------------------------*
C     PROGRAMMER:   KEVIN KLUGHART                                     *
C----------------------------------------------------------------------*
C     DATE:         01-17-81 @ 14:00 CDST                              *
C----------------------------------------------------------------------*
C     PURPOSE:      THIS PROGRAM QUEUES A FILE FOR PRINTING ON A       *
C                   REMOTE NODE.  AT PRESENT, THE ONLY NODE SUPPORTED  *
C                   IS THE PDP-11/34 (HASP) WORKSTATION.               *
C----------------------------------------------------------------------*
C     PARAMETERS                                                       *
C----------------------------------------------------------------------*
      PARAMETER BLANK            = ' '            ! DEFINE SPACE CHARACTER
      PARAMETER EXTENSION        = '.LIS'         ! DEFAULT EXTENSION
      PARAMETER FULL             = 'F'X           ! FULL ERROR REPORT BIT MASK
      PARAMETER INPUT            = 1              ! FILE TEST UNIT
      PARAMETER MAXWID           = 80             ! MAXIMUM FILE NAME LENGTH
      PARAMETER ONE              = 1              ! USEFUL CONSTANT
      PARAMETER PERIOD           = '.'            ! PERIOD - EXTENSION
      PARAMETER PRINTER_MAILBOX  = 'HASP_PRT_MBX' ! REMOTE PRINTER MAILBOX
      PARAMETER RIGHT            = ']'            ! RIGHT BRACKET - USER NAME
      PARAMETER SEMICOLON        = ';'            ! SEMICOLON - VERSION
      PARAMETER ZERO             = 0              ! USEFUL CONSTANT
C----------------------------------------------------------------------*
C     EXTERNAL REFERENCES                                              *
C----------------------------------------------------------------------*
      EXTERNAL IO$_WRITEVBLK
      EXTERNAL IO$M_NOW
      EXTERNAL RMS$_FNF
      EXTERNAL RMS$_NMF
      EXTERNAL RMS$_NORMAL
      EXTERNAL SS$_NORMAL
C----------------------------------------------------------------------*
C     CHARACTER                                                        *
C----------------------------------------------------------------------*
      CHARACTER*(MAXWID) ERRMSG
      CHARACTER*(MAXWID) PRINTER_FILE
C----------------------------------------------------------------------*
C     INTEGER*4                                                        *
C----------------------------------------------------------------------*
      INTEGER*4 LENMSG
      INTEGER*4 LENGTH
      INTEGER*4 LIB$GET_FOREIGN
      INTEGER*4 LIB$PUT_OUTPUT
      INTEGER*4 LIMIT
      INTEGER*4 POINTER
      INTEGER*4 PRINTER_CHANNEL
      INTEGER*4 RMS$PARSE
      INTEGER*4 RMS$SEARCH
      INTEGER*4 STATUS
      INTEGER*4 SYS$ASSIGN
      INTEGER*4 SYS$QIO
      INTEGER*4 WRITE_MAILBOX
C----------------------------------------------------------------------*
C     SECTION 100                                                      *
C----------------------------------------------------------------------*
C     INITIALIZATION                                                   *
C----------------------------------------------------------------------*
C     ESTABLISH COMMUNICATIONS LINK TO RJE/HASP SUBMISSION MAILBOX     *
C----------------------------------------------------------------------*
  100 CONTINUE
      STATUS=SYS$ASSIGN(PRINTER_MAILBOX,PRINTER_CHANNEL,,)

      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 800
C----------------------------------------------------------------------*
C     SECTION 200                                                      *
C----------------------------------------------------------------------*
C     DETERMINE AND VALIDATE JOB FILE NAME                             *
C----------------------------------------------------------------------*
  200 CONTINUE
      STATUS=LIB$GET_FOREIGN(PRINTER_FILE,'Enter File Name: ',LIMIT)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 800
      IF (LIMIT .EQ. ZERO) GO TO 200
C----------------------------------------------------------------------*
C     SET DEFAULT FILE EXTENSION IF NOT SET BY USER                    *
C----------------------------------------------------------------------*
      POINTER=INDEX(PRINTER_FILE,SEMICOLON)
      IF (POINTER .NE. ZERO) GO TO 220

      DO POINTER=LENGTH(PRINTER_FILE),ONE,-ONE
           IF (PRINTER_FILE(POINTER:POINTER) .EQ. RIGHT) GO TO 210
           IF (PRINTER_FILE(POINTER:POINTER) .EQ. PERIOD) GO TO 220
      ENDDO

  210 CONTINUE
      PRINTER_FILE((LENGTH(PRINTER_FILE)+ONE):)=EXTENSION
C----------------------------------------------------------------------*
C     USER HAS NOW SPECIFIED FILENAME - NOW CONVERT TO FULLY QUALIFIED *
C     AND PARSE USER FILE SPECIFICATION, CHECKING FOR SYNTAX ERRORS.   *
C----------------------------------------------------------------------*
  220 CONTINUE
      STATUS=RMS$PARSE(PRINTER_FILE)
      IF (STATUS .NE. %LOC(RMS$_NORMAL)) GO TO 800
C----------------------------------------------------------------------*
C     SECTION 300                                                      *
C----------------------------------------------------------------------*
C     WRITE FILENAME TO BE PRINTED TO THE PRINT QUEUE MAILBOX          *
C----------------------------------------------------------------------*
  300 CONTINUE
      STATUS=RMS$SEARCH(PRINTER_FILE)
      IF (STATUS .EQ. %LOC(RMS$_FNF)) GO TO 800
      IF (STATUS .EQ. %LOC(RMS$_NMF)) GO TO 900
      LIMIT=LENGTH(PRINTER_FILE)

      WRITE_MAILBOX=(%LOC(IO$_WRITEVBLK) .OR. %LOC(IO$M_NOW))
      STATUS=SYS$QIO(,%VAL(PRINTER_CHANNEL),%VAL(WRITE_MAILBOX),,,,
     1     %REF(PRINTER_FILE),%VAL(LIMIT),,,,)

      CALL LIB$PUT_OUTPUT('File '//PRINTER_FILE(ONE:LIMIT)//
     1     ' entered on SPOOL queue')
      IF (STATUS .EQ. %LOC(SS$_NORMAL)) GO TO 300
C----------------------------------------------------------------------*
C     SECTION 800                                                      *
C----------------------------------------------------------------------*
C     ERROR TRAPS                                                      *
C----------------------------------------------------------------------*
C     VAX/VMS SYSTEM ERROR TRAP                                        *
C----------------------------------------------------------------------*
  800 CONTINUE
      CALL SYS$GETMSG(%VAL(STATUS),LENMSG,ERRMSG,%VAL(FULL),)
      CALL LIB$PUT_OUTPUT(ERRMSG(ONE:LENMSG))
C----------------------------------------------------------------------*
C     SECTION 900                                                      *
C----------------------------------------------------------------------*
C     PROGRAM TERMINATION                                              *
C----------------------------------------------------------------------*
  900 CONTINUE
C----------------------------------------------------------------------*
C     END OF MAIN PROGRAM SPOOL_QUEUE                                  *
C----------------------------------------------------------------------*
      END
