      PROGRAM SUSPEND
C----------------------------------------------------------------------*
C     PROGRAM:      SUSPEND                                            *
C----------------------------------------------------------------------*
C     LANGUAGE:     VAX-11 FORTRAN V2.0-2                              *
C     SYSTEM:       VAX-11/780                                         *
C                   MOSTEK CORPORATION                                 *
C                   COMPUTER AIDS TO DESIGN DEPARTMENT                 *
C                   1215 WEST CROSBY ROAD                              *
C                   CARROLLTON, TEXAS  75006                           *
C                   (214)  323-8813                                    *
C----------------------------------------------------------------------*
C     PROGRAMMER:   KEVIN KLUGHART                                     *
C----------------------------------------------------------------------*
C     DATE:         04-05-81 @ 13:00 CDST                              *
C----------------------------------------------------------------------*
C     PURPOSE:      THIS PROGRAM SUSPENDS A GIVEN PROCESS BASED ONE    *
C                   EITHER PROCESS NAME OR HEX PROCESS IDENTIFICATION. *
C----------------------------------------------------------------------*
C     EXTERNALS:    STRING LIBRARY                                     *
C                   SYS$JPI PROCESS INFORMATION SUBROUTINE             *
C----------------------------------------------------------------------*

C----------------------------------------------------------------------*
C                                                                      *
C     PARAMETERS                                                       *
C                                                                      *
C----------------------------------------------------------------------*
C     NUMERIC PARAMETERS                                               *
C----------------------------------------------------------------------*
C     USEFUL CONSTANTS                                                 *
C----------------------------------------------------------------------*
      PARAMETER EIGHT            = 8              ! USEFUL CONSTANT
      PARAMETER FULL             = 'F'X           ! FULL ERROR REPORT
      PARAMETER MAXNAM           = 15             ! MAX PROCESS NAME LENGTH
      PARAMETER NINE             = 9              ! USEFUL CONSTANT
      PARAMETER ONE              = 1              ! USEFUL CONSTANT
      PARAMETER WIDTH            = 80             ! COMMAND INPUT LIMIT
      PARAMETER WILDCARD         = -1             ! WILDCARD PROCESS SEARCH
      PARAMETER ZERO             = 0              ! USEFUL CONSTANT

C----------------------------------------------------------------------*
C                                                                      *
C     EXTERNALS                                                        *
C                                                                      *
C----------------------------------------------------------------------*
      EXTERNAL JPI$_PID
      EXTERNAL JPI$_PRCNAM
      EXTERNAL JPI$_STATE
      EXTERNAL JPI$_TERMINAL
      EXTERNAL SCH$C_SUSP
      EXTERNAL SCH$C_SUSPO
      EXTERNAL SS$_NOMOREPROC
      EXTERNAL SS$_NONEXPR
      EXTERNAL SS$_NORMAL

C----------------------------------------------------------------------*
C                                                                      *
C     CHARACTER                                                        *
C                                                                      *
C----------------------------------------------------------------------*
      CHARACTER*(WIDTH) OUTPUT
      CHARACTER*(WIDTH) PROCESS

C----------------------------------------------------------------------*
C                                                                      *
C     INTEGER*4                                                        *
C                                                                      *
C----------------------------------------------------------------------*
      INTEGER*4 JPI$_PID
      INTEGER*4 JPI$_PRCNAM
      INTEGER*4 JPI$_STATE
      INTEGER*4 JPI$_TERMINAL
      INTEGER*4 LENGTH
      INTEGER*4 LIB$PUT_OUTPUT
      INTEGER*4 LIM
      INTEGER*4 LIMIT
      INTEGER*4 PID
      INTEGER*4 PID_SEARCH
      INTEGER*4 SCH$C_SUSP
      INTEGER*4 SCH$C_SUSPO
      INTEGER*4 SS$_NOMOREPROC
      INTEGER*4 SS$_NONEXPR
      INTEGER*4 SS$_NORMAL
      INTEGER*4 STATE
      INTEGER*4 STATUS
      INTEGER*4 SYS$BRDCST
      INTEGER*4 SYS$JPI
      INTEGER*4 SYS$SUSPND

C----------------------------------------------------------------------*
C                                                                      *
C     LOGICAL*4                                                        *
C                                                                      *
C----------------------------------------------------------------------*
      LOGICAL*4 ERROR
      LOGICAL*4 NODATA

C----------------------------------------------------------------------*
C     SECTION 100                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     INITIALIZE TERMINAL INTERFACE AND REQUEST PROCESS NAME           *
C                                                                      *
C----------------------------------------------------------------------*
  100 CONTINUE
      STATUS=LIB$GET_FOREIGN(PROCESS,'Enter Process Name: ',LIM)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 800
      IF (LIM .EQ. ZERO) GO TO 100
      CALL DELETE_CONTROL(PROCESS)
      LIM=LENGTH(PROCESS)

C----------------------------------------------------------------------*
C     SECTION 200                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     PERFORM PROCESS WILDCARD SEARCH AND TRY TO MATCH PROCESS NAME    *
C     WITH THAT SPECIFIED BY THE USER.  ASSUME USER PROCESS NAME.      *
C                                                                      *
C----------------------------------------------------------------------*
  200 CONTINUE
      PID_SEARCH=WILDCARD

  210 CONTINUE
      STATUS=SYS$JPI(PID_SEARCH,,%LOC(JPI$_PID),PID,)
      IF (STATUS .EQ. %LOC(SS$_NOMOREPROC)) GO TO 300
      STATUS=SYS$JPI(PID,,%LOC(JPI$_PRCNAM),OUTPUT,)
      CALL DELETE_CONTROL(OUTPUT)
      LIMIT=LENGTH(OUTPUT)
      IF (LIM .NE. LIMIT) GO TO 210
      IF (PROCESS(ONE:LIM) .NE. OUTPUT(ONE:LIM)) GO TO 210

      GO TO 400

C----------------------------------------------------------------------*
C     SECTION 300                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     ASSUME USER ENTERED PROCESS ID (HEX) RATHER THAN PROCESS NAME    *
C                                                                      *
C----------------------------------------------------------------------*
  300 CONTINUE
      CALL DECODE_HEXDEC(PROCESS,PID,NODATA,ERROR)
      STATUS=%LOC(SS$_NONEXPR)
      IF (NODATA .OR. ERROR) GO TO 800

      STATUS=SYS$JPI(PID,,%LOC(JPI$_PRCNAM),PROCESS,LIM)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 800

C----------------------------------------------------------------------*
C     SECTION 400                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SUSPEND PROCESS UNLESS PROCESS IS ALREADY IN SUSPENDED STATE     *
C                                                                      *
C----------------------------------------------------------------------*
C     DO NOTHING IF PROCESS IS IN SUSPENDED STATE                      *
C----------------------------------------------------------------------*
  400 CONTINUE
      STATUS=SYS$JPI(PID,,%LOC(JPI$_STATE),STATE,)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 800

      IF ((STATE .EQ. %LOC(SCH$C_SUSP)) .OR.
     1     (STATE .EQ. %LOC(SCH$C_SUSPO))) GO TO 900
C----------------------------------------------------------------------*
C     BROADCAST MESSAGE TO USER IF TERMINAL AVAILABLE                  *
C----------------------------------------------------------------------*
      STATUS=SYS$JPI(PID,,%LOC(JPI$_TERMINAL),OUTPUT,LIMIT)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 800

      IF (LIMIT .EQ. ZERO) GO TO 410
      STATUS=SYS$BRDCST('%SYS-I-SUSPEND:  PROCESS SUSPENDED',
     1     OUTPUT(ONE:LIMIT))
C----------------------------------------------------------------------*
C     SUSPEND PROCESS                                                  *
C----------------------------------------------------------------------*
  410 CONTINUE
      STATUS=SYS$SUSPND(PID,)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 800

      CALL HEXFMT(PID,OUTPUT,EIGHT)
      CALL DELETE_CONTROL(PROCESS)
      STATUS=LIB$PUT_OUTPUT(OUTPUT(ONE:NINE)//PROCESS(ONE:MAXNAM)//
     1     '     ***  SUCCESSFULLY SUSPENDED  ***')
      GO TO 900

C----------------------------------------------------------------------*
C     SECTION 800                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     ERROR PROCESSING                                                 *
C                                                                      *
C----------------------------------------------------------------------*
C     VAX/VMS ERROR REPORTS                                            *
C----------------------------------------------------------------------*
  800 CONTINUE
      CALL SYS$GETMSG(%VAL(STATUS),LIM,OUTPUT,%VAL(FULL),)
      STATUS=LIB$PUT_OUTPUT(OUTPUT(ONE:LIM))

C----------------------------------------------------------------------*
C     SECTION 900                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     PROGRAM TERMINATION                                              *
C                                                                      *
C----------------------------------------------------------------------*
  900 CONTINUE
C----------------------------------------------------------------------*
C     END OF MAIN PROGRAM SUSPEND                                      *
C----------------------------------------------------------------------*
      END

