      FUNCTION CRT_DRIVER
C----------------------------------------------------------------------*
C                                                                      *
C     SUBROUTINE:   CRT_DRIVER                                         *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     LANGUAGE:     VAX-11 FORTRAN IV V2.0-2                           *
C                                                                      *
C                                                                      *
C     SYSTEM:       VAX-11/780                                         *
C                                                                      *
C                                                                      *
C                   MOSTEK CORPORATION                                 *
C                                                                      *
C                                                                      *
C                   COMPUTER AIDED DESIGN DIVISION                     *
C                                                                      *
C                                                                      *
C                   1215 WEST CROSBY ROAD                              *
C                                                                      *
C                                                                      *
C                   CARROLLTON, TEXAS  75006                           *
C                                                                      *
C                                                                      *
C                   (214)  323-8813                                    *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     PROGRAMMER:   KEVIN KLUGHART                                     *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     DATE:         04-24-81 @ 10:00 CDST                              *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     PURPOSE:      THIS IS A SOFTWARE PACKAGE DESIGNED TO PROVIDE A   *
C                                                                      *
C                   UNIFORM AND CONSISTENT METHOD OF DRIVING THE       *
C                                                                      *
C                   SPECIAL FUNCTIONS AVAILABLE ON VARIOUS CRT AND     *
C                                                                      *
C                   HARDCOPY COMPUTER TERMINALS.  BY PROVIDING UNIFORM *
C                                                                      *
C                   AND CONSISTENT FUNCTION ENTRYPOINTS, IT IS HOPED   *
C                                                                      *
C                   TO REACH A POINT WHERE ALL TERMINAL I/O IS         *
C                                                                      *
C                   TRANSPARENT OF THE HARDWARE INVOLVED.  THIS        *
C                                                                      *
C                   UNIVERSAL INTERFACE IS BASED ON ANSI STANDARDS     *
C                                                                      *
C                   X 3.64 1977  AND  X 3.41 1974, BUT WILL SUPPORT    *
C                                                                      *
C                   OTHER NONSTANDARD CONTROL SEQUENCES IF THEY ARE    *
C                                                                      *
C                   GIVEN LOGICAL NAMING CONVENTIONS AND ENTERED IN    *
C                                                                      *
C                   THE DRIVER SYMBOL AND PROGRAM TABLES.              *
C                                                                      *
C----------------------------------------------------------------------*

C----------------------------------------------------------------------*
C                                                                      *
C     EXTERNALS                                                        *
C                                                                      *
C----------------------------------------------------------------------*
C     INTFMT:       PERFORMS INTEGER*4 TO CHARACTER STRING CONVERSION. *
C                                                                      *
C     LENGTH:       DETERMINES THE LOGICAL LENGTH OF A CHARACTER       *
C                   VARIABLE (THE LAST NON-BLANK CHARACTER POSITION).  *
C----------------------------------------------------------------------*

C----------------------------------------------------------------------*
C                                                                      *
C     VAX/VMS QIO DATA STRUCTURES                                      *
C                                                                      *
C----------------------------------------------------------------------*
C     THE GENERAL FORMAT FOR THE SYS$QIO AND SYS$QIOW SYSTEM SERVICE   *
C     CALLS IS AS FOLLOWS:                                             *
C                                                                      *
C     SYS$QIO[W] [EFN],CHAN,                                           *
C                FUNC,                                                 *
C                [IOSB],[ASTADR],[ASTPRM],                             *
C                [P1],[P2],[P3],[P4],[P5],[P6]                         *
C----------------------------------------------------------------------*

C----------------------------------------------------------------------*
C                                                                      *
C     PARAMETER MAP                                                    *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C         C R T   C O M M A N D   D A T A   S T R U C T U R E S        *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     NUMERIC PARAMETERS                                               *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER BYTE             = 1              ! DEFINE BYTE OFFSET
      PARAMETER BYT3             = 3              ! DEFINE 3 BYTE OFFSET
      PARAMETER FIVE             = 5              ! USEFUL CONSTANT
      PARAMETER FOUR             = 4              ! USEFUL CONSTANT
      PARAMETER FULL             = 'F'X           ! FULL ERROR LOGGING
      PARAMETER LONG             = 4              ! DEFINE LONGWORD OFFSET
      PARAMETER MAXCRT           = 10             ! MAXIMUM # OF DEVICES
      PARAMETER NAM              = 64             ! LOGICAL NAME LENGTH MAX
      PARAMETER ONE              = 1              ! USEFUL CONSTANT
      PARAMETER QUAD             = 8              ! DEFINE QUADWORD LENGTH
      PARAMETER SIX              = 6              ! USEFUL CONSTANT
      PARAMETER SIZ              = 511            ! OUTPUT BUFFER SIZE LIMIT
      PARAMETER TWO              = 2              ! USEFUL CONSTANT
      PARAMETER WORD             = 2              ! DEFINE WORD OFFSET
      PARAMETER ZERO             = 0              ! USEFUL CONSTANT
C----------------------------------------------------------------------*
C                                                                      *
C     LOGICAL UNIT ASSIGNMENT PARAMETERS                               *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER CRT_READER       = 'SYS$COMMAND'  ! SYSTEM LOGNAM FOR INPUT
      PARAMETER CRT_WRITER       = 'SYS$OUTPUT'   ! SYSTEM LOGNAM FOR OUTPUT
      PARAMETER CRT_TERMTYPE     = 'SYS$TERMINAL' ! SPECIFIC TERMINAL TYPE

C----------------------------------------------------------------------*
C                                                                      *
C     GENERAL COMMAND DELIMETERS                                       *
C                                                                      *
C----------------------------------------------------------------------*
C     CHARACTER DEFINITIONS                                            *
C----------------------------------------------------------------------*
      CHARACTER*(*) BLANK
      CHARACTER*(*) BELL
      CHARACTER*(*) BACK_SPACE
      CHARACTER*(*) CR
      CHARACTER*(*) DLM
      CHARACTER*(*) ENQ
      CHARACTER*(*) EOF
      CHARACTER*(*) ESC
      CHARACTER*(*) FF
      CHARACTER*(*) FORM_FEED
      CHARACTER*(*) LF
      CHARACTER*(*) NUL
      CHARACTER*(*) PERIOD
      CHARACTER*(*) SI
      CHARACTER*(*) TAB
      CHARACTER*(*) VT

      CHARACTER*(*) ANS
      CHARACTER*(*) CRLF
C----------------------------------------------------------------------*
C     CHARACTER PARAMETERS                                             *
C----------------------------------------------------------------------*
      PARAMETER (BLANK           = CHAR('20'X)   )! DEFINE SPACE CHARACTER
      PARAMETER (BELL            = CHAR('07'X)   )! ASCII BELL CODE
      PARAMETER (BACK_SPACE      = CHAR('08'X)   )! ASCII BACKSPACE CODE
      PARAMETER (CR              = CHAR('0D'X)   )! ASCII CARRIAGE RETURN CODE
      PARAMETER (DLM             = CHAR('3B'X)   )! ASCII DELIMITER (';')
      PARAMETER (ENQ             = CHAR('05'X)   )! ASCII ENQUIRY
      PARAMETER (EOF             = CHAR('1A'X)   )! ASCII END-OF-FILE (^Z)
      PARAMETER (ESC             = CHAR('1B'X)   )! ASCII ESCAPE CODE
      PARAMETER (FF              = CHAR('0C'X)   )! ASCII FORM FEED CODE
      PARAMETER (FORM_FEED       = CHAR('0C'X)   )! ASCII FORM FEED CODE
      PARAMETER (LF              = CHAR('0A'X)   )! ASCII LINE FEED CODE
      PARAMETER (NUL             = CHAR('00'X)   )! ASCII NULL CODE
      PARAMETER (PERIOD          = CHAR('2E'X)   )! ASCII PERIOD
      PARAMETER (SI              = CHAR('0F'X)   )! ASCII SHIFT IN (SI)
      PARAMETER (TAB             = CHAR('09'X)   )! ASCII TAB CODE
      PARAMETER (VT              = CHAR('0B'X)   )! ASCII VERTICAL TAB

      PARAMETER (ANS             = ESC//'['      )! ANSI PREFIX
      PARAMETER (CRLF            = CR//LF        )! ASCII NEW LINE CODE

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C                       *****    *****    ****                         *
C                         *     *     *  *    *                        *
C                         *     *        *                             *
C                         *      *****   *                             *
C                         *           *  *                             *
C                         *     *     *  *    *                        *
C                       *****    *****    ****                         *
C                                                                      *
C                                                                      *
C                                                                      *
C         *****    ******    *****   *     *  *******  ******          *
C         *    *   *     *     *     *     *  *        *     *         *
C         *     *  *     *     *     *     *  *        *     *         *
C         *     *  ******      *     *     *  *****    ******          *
C         *     *  *  *        *      *   *   *        *  *            *
C         *    *   *   *       *       * *    *        *   *           *
C         *****    *    *    *****      *     *******  *    *          *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C     DATA STRUCTURE TYPE DEFINITIONS                                  *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C     INTEGER*4 PARAMETERS                                             *
C----------------------------------------------------------------------*
      INTEGER*4 ISC_COLUMN_MAX
      INTEGER*4 ISC_COLUMN_ORG
      INTEGER*4 ISC_PLOT_X_MAX
      INTEGER*4 ISC_PLOT_X_ORG
      INTEGER*4 ISC_PLOT_Y_MAX
      INTEGER*4 ISC_PLOT_Y_ORG
      INTEGER*4 ISC_ROW_MAX
      INTEGER*4 ISC_ROW_MIN
C----------------------------------------------------------------------*
C     CHARACTER PARAMETERS                                             *
C----------------------------------------------------------------------*
      CHARACTER*(*) ISC_HOME_UP
      CHARACTER*(*) ISC_HOME_DOWN
      CHARACTER*(*) ISC_MOVE_UP
      CHARACTER*(*) ISC_MOVE_DOWN
      CHARACTER*(*) ISC_MOVE_DOWN1
      CHARACTER*(*) ISC_MOVE_RIGHT
      CHARACTER*(*) ISC_MOVE_LEFT
      CHARACTER*(*) ISC_MOVE
      CHARACTER*(*) ISC_BLINK
      CHARACTER*(*) ISC_BLINKA7_OFF
      CHARACTER*(*) ISC_COLOR_BIAS
      CHARACTER*(*) ISC_COLOR_MASK
      CHARACTER*(*) ISC_FOREGROUND
      CHARACTER*(*) ISC_BACKGROUND
      CHARACTER*(*) ISC_HUE_BLACK
      CHARACTER*(*) ISC_HUE_RED
      CHARACTER*(*) ISC_HUE_GREEN
      CHARACTER*(*) ISC_HUE_YELLOW
      CHARACTER*(*) ISC_HUE_BLUE
      CHARACTER*(*) ISC_HUE_MAGENTA
      CHARACTER*(*) ISC_HUE_CYAN
      CHARACTER*(*) ISC_HUE_WHITE
      CHARACTER*(*) ISC_CLEAR_LINE
      CHARACTER*(*) ISC_CLEAR_PAGE
      CHARACTER*(*) ISC_GRAPHICS
      CHARACTER*(*) ISC_PLOT_ESCAPE
      CHARACTER*(*) ISC_PLOT_CELL
      CHARACTER*(*) ISC_PLOT_XPOINT
      CHARACTER*(*) ISC_PLOT_YPOINT
      CHARACTER*(*) ISC_PLOT_XY
      CHARACTER*(*) ISC_PLOT_XBAR_X
      CHARACTER*(*) ISC_PLOT_XBAR_Y
      CHARACTER*(*) ISC_PLOT_XBAR_M
      CHARACTER*(*) ISC_PLOT_XBAR_I
      CHARACTER*(*) ISC_PLOT_YBAR_Y
      CHARACTER*(*) ISC_PLOT_YBAR_X
      CHARACTER*(*) ISC_PLOT_YBAR_M
      CHARACTER*(*) ISC_PLOT_YBAR_I
      CHARACTER*(*) ISC_PLOT_LINE_X
      CHARACTER*(*) ISC_PLOT_LINE_Y
      CHARACTER*(*) ISC_PLOT_LINE_I
C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C     PARAMETER DEFINITIONS                                            *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     NUMERIC PARAMETERS                                               *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (ISC_COLUMN_MAX  = 79            )! MAXIMUM COLUMN COUNT
      PARAMETER (ISC_COLUMN_ORG  = 0             )! MINIMUM COLUMN COUNT
      PARAMETER (ISC_PLOT_X_MAX  = 159           )! X-AXIS LIMIT
      PARAMETER (ISC_PLOT_X_ORG  = 0             )! X-AXIS ORIGIN
      PARAMETER (ISC_PLOT_Y_MAX  = 191           )! Y-AXIS LIMIT
      PARAMETER (ISC_PLOT_Y_ORG  = 0             )! Y-AXIS ORIGIN
      PARAMETER (ISC_ROW_MAX     = 47            )! MAXIMUM ROW COUNT
      PARAMETER (ISC_ROW_MIN     = 0             )! MINIMUM ROW COUNT
C----------------------------------------------------------------------*
C                                                                      *
C     CURSOR MOVEMENT COMMANDS                                         *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (ISC_HOME_UP     = CHAR('08'X)   )! CURSOR HOME:  SCREEN TOP
      PARAMETER (ISC_HOME_DOWN   = NUL//NUL      )! CURSOR HOME:  SCREEN BOTTOM
      PARAMETER (ISC_MOVE_UP     = CHAR('1C'X)   )! CURSOR UP
      PARAMETER (ISC_MOVE_DOWN   = CHAR('00'X)   )! CURSOR DOWN
      PARAMETER (ISC_MOVE_DOWN1  = CHAR('00'X)   )! CURSOR DOWN ONE LINE
      PARAMETER (ISC_MOVE_RIGHT  = CHAR('19'X)   )! CURSOR FORWARD (RIGHT)
      PARAMETER (ISC_MOVE_LEFT   = CHAR('1A'X)   )! CURSOR BACKWARD (LEFT)
      PARAMETER (ISC_MOVE        = CHAR('03'X)   )! DIRECT CURSOR ADDRESSING
C----------------------------------------------------------------------*
C                                                                      *
C     CHARACTER ATTRIBUTE COMMANDS                                     *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (ISC_BLINK       = CHAR('1F'X)   )! BLINK ON
      PARAMETER (ISC_BLINKA7_OFF = CHAR('0F'X)   )! BLINK/BOLD OFF
C----------------------------------------------------------------------*
C                                                                      *
C     CHARACTER/PLOT COLOR COMMANDS                                    *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (ISC_COLOR_BIAS  = CHAR('10'X)   )! COLOR SELECT OFFSET
      PARAMETER (ISC_COLOR_MASK  = CHAR('07'X)   )! COLOR MASK BITS (MOD 8)
      PARAMETER (ISC_FOREGROUND  = CHAR('1D'X)   )! SELECT FOREGROUND COLOR
      PARAMETER (ISC_BACKGROUND  = CHAR('1E'X)   )! SELECT BACKGROUND COLOR
      PARAMETER (ISC_HUE_BLACK   = CHAR('10'X)   )! SELECT COLOR:  BLACK
      PARAMETER (ISC_HUE_RED     = CHAR('11'X)   )! SELECT COLOR:  RED
      PARAMETER (ISC_HUE_GREEN   = CHAR('12'X)   )! SELECT COLOR:  GREEN
      PARAMETER (ISC_HUE_YELLOW  = CHAR('13'X)   )! SELECT COLOR:  YELLOW
      PARAMETER (ISC_HUE_BLUE    = CHAR('14'X)   )! SELECT COLOR:  BLUE
      PARAMETER (ISC_HUE_MAGENTA = CHAR('15'X)   )! SELECT COLOR:  MAGENTA
      PARAMETER (ISC_HUE_CYAN    = CHAR('16'X)   )! SELECT COLOR:  CYAN
      PARAMETER (ISC_HUE_WHITE   = CHAR('17'X)   )! SELECT COLOR:  WHITE
C----------------------------------------------------------------------*
C                                                                      *
C     ERASING COMMANDS                                                 *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (ISC_CLEAR_LINE  = CHAR('0B'X)   )! ERASE LINE
      PARAMETER (ISC_CLEAR_PAGE  = CHAR('0C'X)   )! ERASE PAGE
C----------------------------------------------------------------------*
C                                                                      *
C     GRAPHIC SYMBOL COMMANDS                                          *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (ISC_GRAPHICS    = CHAR('02'X)   )! ENTER GRAPHICS (PLOT) MODE
      PARAMETER (ISC_PLOT_ESCAPE = CHAR('FF'X)   )! EXIT PLOT MODE
      PARAMETER (ISC_PLOT_CELL   = CHAR('FE'X)   )! CHARACTER MODE PLOT
      PARAMETER (ISC_PLOT_XPOINT = CHAR('FD'X)   )! X-POINT PLOT
      PARAMETER (ISC_PLOT_YPOINT = CHAR('FC'X)   )! Y-POINT PLOT
      PARAMETER (ISC_PLOT_XY     = CHAR('FB'X)   )! X-Y INCREMENTAL PLOT

      PARAMETER (ISC_PLOT_XBAR_X = CHAR('FA'X)   )! X-BAR GRAPH:  X-VALUE
      PARAMETER (ISC_PLOT_XBAR_Y = CHAR('F9'X)   )! X-BAR GRAPH:  Y-VALUE
      PARAMETER (ISC_PLOT_XBAR_M = CHAR('F8'X)   )! X-BAR GRAPH:  Y-MAX-VALUE
      PARAMETER (ISC_PLOT_XBAR_I = CHAR('F7'X)   )! X-BAR GRAPH:  INCREMENTAL

      PARAMETER (ISC_PLOT_YBAR_Y = CHAR('F6'X)   )! Y-BAR GRAPH:  Y-VALUE
      PARAMETER (ISC_PLOT_YBAR_X = CHAR('F5'X)   )! Y-BAR GRAPH:  X-VALUE
      PARAMETER (ISC_PLOT_YBAR_M = CHAR('F4'X)   )! Y-BAR GRAPH:  X-MAX-VALUE
      PARAMETER (ISC_PLOT_YBAR_I = CHAR('F3'X)   )! Y-BAR GRAPH:  INCREMENTAL

      PARAMETER (ISC_PLOT_LINE_X = CHAR('F2'X)   )! VECTOR PLOT:  X-VALUE
      PARAMETER (ISC_PLOT_LINE_Y = CHAR('F1'X)   )! VECTOR PLOT:  Y-VALUE
      PARAMETER (ISC_PLOT_LINE_I = CHAR('F0'X)   )! VECTOR PLOT:  INCREMENTAL

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C         *           *                 *      *****    *****          *
C         *          * *               **     *     *  *     *         *
C         *         *   *             * *           *  *     *         *
C         *        *     *   *****      *      *****   *     *         *
C         *        *******              *     *        *     *         *
C         *        *     *              *     *        *     *         *
C         *******  *     *            *****   ******    *****          *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C         *****    ******    *****   *     *  *******  ******          *
C         *    *   *     *     *     *     *  *        *     *         *
C         *     *  *     *     *     *     *  *        *     *         *
C         *     *  ******      *     *     *  *****    ******          *
C         *     *  *  *        *      *   *   *        *  *            *
C         *    *   *   *       *       * *    *        *   *           *
C         *****    *    *    *****      *     *******  *    *          *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C     DATA STRUCTURE TYPE DEFINITIONS                                  *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C     CHARACTER PARAMETERS                                             *
C----------------------------------------------------------------------*
      CHARACTER*(*) LA1_H_PITCH_05
      CHARACTER*(*) LA1_H_PITCH_06
      CHARACTER*(*) LA1_H_PITCH_07
      CHARACTER*(*) LA1_H_PITCH_08
      CHARACTER*(*) LA1_H_PITCH_10
      CHARACTER*(*) LA1_H_PITCH_12
      CHARACTER*(*) LA1_H_PITCH_13
      CHARACTER*(*) LA1_H_PITCH_16
      CHARACTER*(*) LA1_V_PITCH_02
      CHARACTER*(*) LA1_V_PITCH_03
      CHARACTER*(*) LA1_V_PITCH_04
      CHARACTER*(*) LA1_V_PITCH_06
      CHARACTER*(*) LA1_V_PITCH_08
      CHARACTER*(*) LA1_V_PITCH_12
      CHARACTER*(*) LA1_FORM_LENGTH
      CHARACTER*(*) LA1_MARGIN
      CHARACTER*(*) LA1_REGION
      CHARACTER*(*) LA1_UK
      CHARACTER*(*) LA1_USASCII
      CHARACTER*(*) LA1_APL
      CHARACTER*(*) LA1_FINLAND
      CHARACTER*(*) LA1_DENMARK
      CHARACTER*(*) LA1_NORWAY
      CHARACTER*(*) LA1_SWEDEN
      CHARACTER*(*) LA1_GERMANY
      CHARACTER*(*) LA1_FRANCE
      CHARACTER*(*) LA1_SET_TAB
      CHARACTER*(*) LA1_CLEAR_TAB
      CHARACTER*(*) LA1_CLEAR_TABS
      CHARACTER*(*) LA1_SET_VFU
      CHARACTER*(*) LA1_CLEAR_VFU
      CHARACTER*(*) LA1_CLEAR_VFUS
      CHARACTER*(*) LA1_NL_MODE
      CHARACTER*(*) LA1_LF_MODE
      CHARACTER*(*) LA1_KEYPAD_APP
      CHARACTER*(*) LA1_KEYPAD_NUM
      CHARACTER*(*) LA1_ANSWERBACK
C----------------------------------------------------------------------*
C                                                                      *
C     HORIZONTAL AND VERTICAL PITCH COMMANDS                           *
C                                                                      *
C----------------------------------------------------------------------*
C     HORIZONTAL PITCH COMMANDS                                        *
C----------------------------------------------------------------------*
      PARAMETER (LA1_H_PITCH_05  = ANS//'5w'     )! 5 CHAR/INCH
      PARAMETER (LA1_H_PITCH_06  = ANS//'6w'     )! 6 CHAR/INCH
      PARAMETER (LA1_H_PITCH_07  = ANS//'7w'     )! 6.6 CHAR/INCH
      PARAMETER (LA1_H_PITCH_08  = ANS//'8w'     )! 8.25 CHAR/INCH
      PARAMETER (LA1_H_PITCH_10  = ANS//'w'      )! 10 CHAR/INCH
      PARAMETER (LA1_H_PITCH_12  = ANS//'2w'     )! 12 CHAR/INCH
      PARAMETER (LA1_H_PITCH_13  = ANS//'3w'     )! 13.2 CHAR/INCH
      PARAMETER (LA1_H_PITCH_16  = ANS//'4w'     )! 16.5 CHAR/INCH
C----------------------------------------------------------------------*
C     VERTICAL PITCH COMMANDS                                          *
C----------------------------------------------------------------------*
      PARAMETER (LA1_V_PITCH_02  = ANS//'4z'     )! 2 LINES/INCH
      PARAMETER (LA1_V_PITCH_03  = ANS//'5z'     )! 3 LINES/INCH
      PARAMETER (LA1_V_PITCH_04  = ANS//'6z'     )! 4 LINES/INCH
      PARAMETER (LA1_V_PITCH_06  = ANS//'z'      )! 6 LINES/INCH
      PARAMETER (LA1_V_PITCH_08  = ANS//'2z'     )! 8 LINES/INCH
      PARAMETER (LA1_V_PITCH_12  = ANS//'3z'     )! 12 LINES/INCH
C----------------------------------------------------------------------*
C                                                                      *
C     HORIZONTAL AND VERTICAL MARGIN COMMANDS                          *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (LA1_FORM_LENGTH = 't'           )! VERTICAL FORM LENGTH
      PARAMETER (LA1_MARGIN      = 's'           )! HORIZONTAL MARGIN
      PARAMETER (LA1_REGION      = 'r'           )! VERTICAL MARGIN
C----------------------------------------------------------------------*
C                                                                      *
C     CHARACTER SET COMMANDS                                           *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (LA1_UK          = ESC//'(A'     )! GREAT BRITAIN
      PARAMETER (LA1_USASCII     = ESC//'(B'     )! UNITED STATES ASCII
      PARAMETER (LA1_APL         = SI            )! APL CHARACTER SET (SI)
      PARAMETER (LA1_FINLAND     = ESC//'(C'     )! FINLAND
      PARAMETER (LA1_DENMARK     = ESC//'(E'     )! NORWAY/DENMARK
      PARAMETER (LA1_NORWAY      = ESC//'(E'     )! NORWAY/DENMARK
      PARAMETER (LA1_SWEDEN      = ESC//'(H'     )! SWEDEN
      PARAMETER (LA1_GERMANY     = ESC//'(K'     )! GERMANY
      PARAMETER (LA1_FRANCE      = ESC//'(R'     )! FRANCE
C----------------------------------------------------------------------*
C                                                                      *
C     TAB STOP COMMANDS                                                *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (LA1_SET_TAB     = ESC//'H'      )! SET TAB AT CURRENT COLUMN
      PARAMETER (LA1_CLEAR_TAB   = ANS//'g'      )! CLEAR TAB AT CURRENT COLUMN
      PARAMETER (LA1_CLEAR_TABS  = ANS//'3g'     )! CLEAR ALL TABS
      PARAMETER (LA1_SET_VFU     = ESC//'J'      )! SET VFU AT CURRENT COLUMN
      PARAMETER (LA1_CLEAR_VFU   = ANS//'1g'     )! CLEAR VFU AT CURRENT COLUMN
      PARAMETER (LA1_CLEAR_VFUS  = ANS//'4g'     )! CLEAR ALL VFUS
C----------------------------------------------------------------------*
C                                                                      *
C     MODE CONVERSION COMMANDS                                         *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (LA1_NL_MODE     = ANS//'20h'    )! NEW LINE MODE
      PARAMETER (LA1_LF_MODE     = ANS//'20l'    )! LINE FEED MODE
      PARAMETER (LA1_KEYPAD_APP  = ESC//'='      )! KEYPAD APPLICATION MODE
      PARAMETER (LA1_KEYPAD_NUM  = ESC//'>'      )! KEYPAD NUMERIC MODE
C----------------------------------------------------------------------*
C                                                                      *
C     DEVICE STATUS REPORT COMMANDS                                    *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (LA1_ANSWERBACK  = ENQ           )! ANSWERBACK ENQUIRY

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C             *     *  *******           *******   *****               *
C             *     *     *              *        *     *              *
C             *     *     *              ******         *              *
C             *     *     *      *****         *   *****               *
C              *   *      *                    *  *                    *
C               * *       *              *     *  *                    *
C                *        *               *****   ******               *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C         *****    ******    *****   *     *  *******  ******          *
C         *    *   *     *     *     *     *  *        *     *         *
C         *     *  *     *     *     *     *  *        *     *         *
C         *     *  ******      *     *     *  *****    ******          *
C         *     *  *  *        *      *   *   *        *  *            *
C         *    *   *   *       *       * *    *        *   *           *
C         *****    *    *    *****      *     *******  *    *          *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C     DATA STRUCTURE TYPE DEFINITIONS                                  *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C     INTEGER*4 PARAMETERS                                             *
C----------------------------------------------------------------------*
      INTEGER*4 VT5_COLUMN_MAX
      INTEGER*4 VT5_COLUMN_ORG
      INTEGER*4 VT5_ROW_MAX
      INTEGER*4 VT5_ROW_ORG
C----------------------------------------------------------------------*
C     CHARACTER PARAMETERS                                             *
C----------------------------------------------------------------------*
      CHARACTER*(*) VT5_HOME_UP
      CHARACTER*(*) VT5_HOME_RIGHT
      CHARACTER*(*) VT5_HOME_DOWN
      CHARACTER*(*) VT5_HOME_BOTTOM
      CHARACTER*(*) VT5_MOVE_UP
      CHARACTER*(*) VT5_MOVE_DOWN
      CHARACTER*(*) VT5_MOVE_RIGHT
      CHARACTER*(*) VT5_MOVE_LEFT
      CHARACTER*(*) VT5_INDEX_DOWN
      CHARACTER*(*) VT5_CLEAR_EOL
      CHARACTER*(*) VT5_CLEAR_EOP
      CHARACTER*(*) VT5_USASCII
      CHARACTER*(*) VT5_GRAPHICS
      CHARACTER*(*) VT5_KEYPAD_APP
      CHARACTER*(*) VT5_KEYPAD_NUM
      CHARACTER*(*) VT5_ANSI
C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C     PARAMETER DEFINITIONS                                            *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     NUMERIC PARAMETERS                                               *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT5_COLUMN_MAX  = 80            )! MAXIMUM COLUMN
      PARAMETER (VT5_COLUMN_ORG  = '1F'X         )! COLUMN ORIGIN
      PARAMETER (VT5_ROW_MAX     = 24            )! MAXIMUM ROW
      PARAMETER (VT5_ROW_ORG     = '1F'X         )! ROW ORIGIN
C----------------------------------------------------------------------*
C                                                                      *
C     CHARACTER PARAMETERS                                             *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     CURSOR MOVEMENT COMMANDS                                         *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT5_HOME_UP     = ESC//'H'      )! CURSOR HOME: SCREEN TOP
      PARAMETER (VT5_HOME_RIGHT  = ESC//'o '     )! CURSOR HOME: SCREEN URC
      PARAMETER (VT5_HOME_DOWN   = ESC//'7 '     )! CURSOR HOME: SCREEN BOTTOM
      PARAMETER (VT5_HOME_BOTTOM = ESC//'o7'     )! CURSOR HOME: SCREEN LRC
      PARAMETER (VT5_MOVE_UP     = ESC//'A'      )! CURSOR UP
      PARAMETER (VT5_MOVE_DOWN   = ESC//'B'      )! CURSOR DOWN
      PARAMETER (VT5_MOVE_RIGHT  = ESC//'C'      )! CURSOR RIGHT
      PARAMETER (VT5_MOVE_LEFT   = ESC//'D'      )! CURSOR LEFT
      PARAMETER (VT5_INDEX_DOWN  = ESC//'I'      )! INDEX
C----------------------------------------------------------------------*
C                                                                      *
C     ERASING COMMANDS                                                 *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT5_CLEAR_EOL   = ESC//'K'      )! ERASE TO END-OF-LINE
      PARAMETER (VT5_CLEAR_EOP   = ESC//'J'      )! ERASE TO END-OF-PAGE
C----------------------------------------------------------------------*
C                                                                      *
C     CHARACTER SET DESIGNATOR COMMANDS                                *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT5_USASCII     = ESC//'G'      )! UNITED STATES (USASCII)
      PARAMETER (VT5_GRAPHICS    = ESC//'F'      )! SPECIAL GRAPHICS 
C----------------------------------------------------------------------*
C                                                                      *
C     MODE CONVERSION COMMANDS                                         *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT5_KEYPAD_APP  = ESC//'='      )! KEYPAD APPLICATION MODE
      PARAMETER (VT5_KEYPAD_NUM  = ESC//'>'      )! KEYPAD NUMERIC MODE
      PARAMETER (VT5_ANSI        = ESC//'<'      )! ENTER ANSI MODE

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C         *     *  *******              *      *****    *****          *
C         *     *     *                **     *     *  *     *         *
C         *     *     *               * *     *     *  *     *         *
C         *     *     *      *****      *     *     *  *     *         *
C          *   *      *                 *     *     *  *     *         *
C           * *       *                 *     *     *  *     *         *
C            *        *               *****    *****    *****          *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C         *****    ******    *****   *     *  *******  ******          *
C         *    *   *     *     *     *     *  *        *     *         *
C         *     *  *     *     *     *     *  *        *     *         *
C         *     *  ******      *     *     *  *****    ******          *
C         *     *  *  *        *      *   *   *        *  *            *
C         *    *   *   *       *       * *    *        *   *           *
C         *****    *    *    *****      *     *******  *    *          *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C     DATA STRUCTURE TYPE DEFINITIONS                                  *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C     INTEGER*4 PARAMETERS                                             *
C----------------------------------------------------------------------*
      INTEGER*4 VT1_COLUMN_MAX
      INTEGER*4 VT1_COLUMN_ORG
      INTEGER*4 VT1_ROW_MAX
      INTEGER*4 VT1_ROW_ORG
      INTEGER*4 VT1_SCAN_LINES
      INTEGER*4 VT1_WIDE_WIDTH
C----------------------------------------------------------------------*
C     CHARACTER PARAMETERS                                             *
C----------------------------------------------------------------------*
      CHARACTER*(*) VT1_HOME_UP
      CHARACTER*(*) VT1_HOME_RIGHT
      CHARACTER*(*) VT1_HOME_DOWN
      CHARACTER*(*) VT1_HOME_BOTTOM
      CHARACTER*(*) VT1_MOVE_UP
      CHARACTER*(*) VT1_MOVE_DOWN
      CHARACTER*(*) VT1_MOVE_DOWN1
      CHARACTER*(*) VT1_MOVE_RIGHT
      CHARACTER*(*) VT1_MOVE_LEFT
      CHARACTER*(*) VT1_MOVE
      CHARACTER*(*) VT1_INDEX_DOWN
      CHARACTER*(*) VT1_INDEX_NEXT
      CHARACTER*(*) VT1_INDEX_UP
      CHARACTER*(*) VT1_STORE
      CHARACTER*(*) VT1_RESTORE
      CHARACTER*(*) VT1_DOUBLE_TOP
      CHARACTER*(*) VT1_DOUBLE_BOT
      CHARACTER*(*) VT1_SINGLE_WIDE
      CHARACTER*(*) VT1_DOUBLE_WIDE
      CHARACTER*(*) VT1_TURN_OFF
      CHARACTER*(*) VT1_BOLD
      CHARACTER*(*) VT1_UNDER
      CHARACTER*(*) VT1_BLINK
      CHARACTER*(*) VT1_REVERSE
      CHARACTER*(*) VT1_CLEAR_EOL
      CHARACTER*(*) VT1_CLEAR_BOL
      CHARACTER*(*) VT1_CLEAR_LINE
      CHARACTER*(*) VT1_CLEAR_EOP
      CHARACTER*(*) VT1_CLEAR_BOP
      CHARACTER*(*) VT1_CLEAR_PAGE
      CHARACTER*(*) VT1_SET_LED
      CHARACTER*(*) VT1_CLEAR_LEDS
      CHARACTER*(*) VT1_UK_G0
      CHARACTER*(*) VT1_UK_G1
      CHARACTER*(*) VT1_USASCII_G0
      CHARACTER*(*) VT1_USASCII_G1
      CHARACTER*(*) VT1_GRAPHICS_G0
      CHARACTER*(*) VT1_GRAPHICS_G1
      CHARACTER*(*) VT1_ROM1_G0
      CHARACTER*(*) VT1_ROM1_G1
      CHARACTER*(*) VT1_ROM2_G0
      CHARACTER*(*) VT1_ROM2_G1
      CHARACTER*(*) VT1_SYM_BLANK
      CHARACTER*(*) VT1_SYM_DIAMOND
      CHARACTER*(*) VT1_SYM_CHECK
      CHARACTER*(*) VT1_SYM_HT
      CHARACTER*(*) VT1_SYM_FF
      CHARACTER*(*) VT1_SYM_CR
      CHARACTER*(*) VT1_SYM_LF
      CHARACTER*(*) VT1_SYM_DEGREE
      CHARACTER*(*) VT1_SYM_PL_MIN
      CHARACTER*(*) VT1_SYM_NL
      CHARACTER*(*) VT1_SYM_VT
      CHARACTER*(*) VT1_SYM_LRC
      CHARACTER*(*) VT1_SYM_URC
      CHARACTER*(*) VT1_SYM_ULC
      CHARACTER*(*) VT1_SYM_LLC
      CHARACTER*(*) VT1_SYM_CROSS
      CHARACTER*(*) VT1_SYM_SCAN1
      CHARACTER*(*) VT1_SYM_SCAN3
      CHARACTER*(*) VT1_SYM_SCAN5
      CHARACTER*(*) VT1_SYM_SCAN7
      CHARACTER*(*) VT1_SYM_SCAN9
      CHARACTER*(*) VT1_SYM_LEFT_T
      CHARACTER*(*) VT1_SYM_RIGHT_T
      CHARACTER*(*) VT1_SYM_LOWER_T
      CHARACTER*(*) VT1_SYM_UPPER_T
      CHARACTER*(*) VT1_SYM_V_BAR
      CHARACTER*(*) VT1_SYM_LE
      CHARACTER*(*) VT1_SYM_GE
      CHARACTER*(*) VT1_SYM_PI
      CHARACTER*(*) VT1_SYM_NE
      CHARACTER*(*) VT1_SYM_POUND
      CHARACTER*(*) VT1_SYM_DOT
      CHARACTER*(*) VT1_SET_REGION
      CHARACTER*(*) VT1_RST_REGION
      CHARACTER*(*) VT1_SET_TAB
      CHARACTER*(*) VT1_CLEAR_TAB
      CHARACTER*(*) VT1_CLEAR_TABS
      CHARACTER*(*) SET
      CHARACTER*(*) RST
      CHARACTER*(*) VT1_NL_MODE
      CHARACTER*(*) VT1_LF_MODE
      CHARACTER*(*) VT1_CURSOR_APP
      CHARACTER*(*) VT1_CURSOR_KEY
      CHARACTER*(*) VT1_ANSI
      CHARACTER*(*) VT1_VT52
      CHARACTER*(*) VT1_132_COLUMN
      CHARACTER*(*) VT1_80_COLUMN
      CHARACTER*(*) VT1_SMOOTH
      CHARACTER*(*) VT1_JUMP
      CHARACTER*(*) VT1_PAGE_LIGHT
      CHARACTER*(*) VT1_PAGE_DARK
      CHARACTER*(*) VT1_RELATIVE
      CHARACTER*(*) VT1_ABSOLUTE
      CHARACTER*(*) VT1_WRAP_ON
      CHARACTER*(*) VT1_WRAP_OFF
      CHARACTER*(*) VT1_REPEAT_ON
      CHARACTER*(*) VT1_REPEAT_OFF
      CHARACTER*(*) VT1_RETRACE_ON
      CHARACTER*(*) VT1_RETRACE_OFF
      CHARACTER*(*) VT1_GRAPHIC_ON
      CHARACTER*(*) VT1_GRAPHIC_OFF
      CHARACTER*(*) VT1_KEYPAD_APP
      CHARACTER*(*) VT1_KEYPAD_NUM
      CHARACTER*(*) VT1_POSITION
      CHARACTER*(*) VT1_POS_REPORT
      CHARACTER*(*) VT1_STATUS
      CHARACTER*(*) VT1_ATTRIBUTES
      CHARACTER*(*) VT1_REQUEST
      CHARACTER*(*) VT1_ANSWERBACK
      CHARACTER*(*) VT1_RESET
      CHARACTER*(*) VT1_ALIGNMENT
      CHARACTER*(*) VT1_TEST
      CHARACTER*(*) VT1_TEST_ENDFIX
      CHARACTER*(*) VT1_PRINT_PAGE
      CHARACTER*(*) VT1_PRINTER_OFF
      CHARACTER*(*) VT1_PRINTER_ON
C----------------------------------------------------------------------*
C                                                                      *
C     NUMERIC PARAMETERS                                               *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_COLUMN_MAX  = 80            )! 80 COLUMN MODE WIDTH
      PARAMETER (VT1_COLUMN_ORG  = 1             )! FIRST COLUMN ON PAGE
      PARAMETER (VT1_ROW_MAX     = 24            )! LAST ROW ON PAGE
      PARAMETER (VT1_ROW_ORG     = 1             )! FIRST ROW ON PAGE
      PARAMETER (VT1_SCAN_LINES  = 5             )! NUMBER OF POINTS/CHARACTER
      PARAMETER (VT1_WIDE_WIDTH  = 132           )! 132 COLUMN MODE WIDTH
C----------------------------------------------------------------------*
C                                                                      *
C     CURSOR MOVEMENT COMMANDS                                         *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_HOME_UP     = ANS//';H'     )! CURSOR HOME:  SCREEN TOP
      PARAMETER (VT1_HOME_RIGHT  = ANS//'1;132H' )! CURSOR HOME:  SCREEN URC
      PARAMETER (VT1_HOME_DOWN   = ANS//'24;H'   )! CURSOR HOME:  SCREEN BOTTOM
      PARAMETER (VT1_HOME_BOTTOM = ANS//'24;132H')! CURSOR HOME:  SCREEN LRC
      PARAMETER (VT1_MOVE_UP     = 'A'           )! CURSOR UP
      PARAMETER (VT1_MOVE_DOWN   = 'B'           )! CURSOR DOWN
      PARAMETER (VT1_MOVE_DOWN1  = ANS//'1B'     )! CURSOR DOWN ONE LINE
      PARAMETER (VT1_MOVE_RIGHT  = 'C'           )! CURSOR FORWARD (RIGHT)
      PARAMETER (VT1_MOVE_LEFT   = 'D'           )! CURSOR BACKWARD (LEFT)
      PARAMETER (VT1_MOVE        = 'H'           )! DIRECT CURSOR ADDRESSING
      PARAMETER (VT1_INDEX_DOWN  = ESC//'D'      )! INDEX
      PARAMETER (VT1_INDEX_NEXT  = ESC//'E'      )! NEXT LINE
      PARAMETER (VT1_INDEX_UP    = ESC//'M'      )! REVERSE INDEX
      PARAMETER (VT1_STORE       = ESC//'7'      )! SAVE CURSOR AND ATTRIBUTES
      PARAMETER (VT1_RESTORE     = ESC//'8'      )! RESTORE CURSOR AND ATTRIBUTES
C----------------------------------------------------------------------*
C                                                                      *
C     HORIZONTAL AND VERTICAL PITCH COMMANDS                           *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_DOUBLE_TOP  = ESC//'#3'     )! DOUBLE-HEIGHT TOP HALF
      PARAMETER (VT1_DOUBLE_BOT  = ESC//'#4'     )! DOUBLE-HEIGHT BOTTOM HALF
      PARAMETER (VT1_SINGLE_WIDE = ESC//'#5'     )! SINGLE-WIDTH SINGLE-HEIGHT
      PARAMETER (VT1_DOUBLE_WIDE = ESC//'#6'     )! DOUBLE-WIDTH SINGLE-HEIGHT
C----------------------------------------------------------------------*
C                                                                      *
C     CHARACTER ATTRIBUTE COMMANDS                                     *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_TURN_OFF    = ANS//'m'      )! ALL ATTRIBUTES OFF
      PARAMETER (VT1_BOLD        = ANS//'1m'     )! BOLD ON
      PARAMETER (VT1_UNDER       = ANS//'4m'     )! UNDERSCORE ON
      PARAMETER (VT1_BLINK       = ANS//'5m'     )! BLINK ON
      PARAMETER (VT1_REVERSE     = ANS//'7m'     )! REVERSE VIDEO ON
C----------------------------------------------------------------------*
C                                                                      *
C     ERASING COMMANDS                                                 *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_CLEAR_EOL   = ANS//'K'      )! ERASE TO END-OF-LINE
      PARAMETER (VT1_CLEAR_BOL   = ANS//'1K'     )! ERASE FROM BEGINNING-OF-LINE
      PARAMETER (VT1_CLEAR_LINE  = ANS//'2K'     )! ERASE LINE
      PARAMETER (VT1_CLEAR_EOP   = ANS//'J'      )! ERASE TO END-OF-PAGE
      PARAMETER (VT1_CLEAR_BOP   = ANS//'1J'     )! ERASE FROM BEGINNING-OF-PAGE
      PARAMETER (VT1_CLEAR_PAGE  = ANS//'2J'     )! ERASE PAGE
C----------------------------------------------------------------------*
C                                                                      *
C     PROGRAMMABLE LIGHT EMITTING DIODE (LED) COMMANDS                 *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_SET_LED     = 'q'           )! LED COMMAND POSTFIX
      PARAMETER (VT1_CLEAR_LEDS  = ANS//DLM//'q' )! ALL LEDS OFF
C----------------------------------------------------------------------*
C                                                                      *
C     CHARACTER SET (G0 AND G1) DESIGNATOR) COMMANDS                   *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_UK_G0       = ESC//'(A'     )! UNITED KINGDOM G0
      PARAMETER (VT1_UK_G1       = ESC//')A'     )! UNITED KINGDOM G1
      PARAMETER (VT1_USASCII_G0  = ESC//'(B'     )! UNITED STATES G0
      PARAMETER (VT1_USASCII_G1  = ESC//')B'     )! UNITED STATES G1
      PARAMETER (VT1_GRAPHICS_G0 = ESC//'(0'     )! GRAPHICS & LINE DRAWING G0
      PARAMETER (VT1_GRAPHICS_G1 = ESC//')0'     )! GRAPHICS & LINE DRAWING G1
      PARAMETER (VT1_ROM1_G0     = ESC//'(1'     )! ALTERNATE CHARACTER ROM G0
      PARAMETER (VT1_ROM1_G1     = ESC//')1'     )! ALTERNATE CHARACTER ROM G1
      PARAMETER (VT1_ROM2_G0     = ESC//'(2'     )! SPECIAL GRAPHICS ROM G0
      PARAMETER (VT1_ROM2_G1     = ESC//')2'     )! SPECIAL GRAPHICS ROM G1
C----------------------------------------------------------------------*
C                                                                      *
C     GRAPHIC SYMBOL COMMANDS                                          *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_SYM_BLANK   = '_'           )! BLANK
      PARAMETER (VT1_SYM_DIAMOND = '`'           )! DIAMOND
      PARAMETER (VT1_SYM_CHECK   = 'a'           )! CHECK BOARD (ERROR INDICATOR)
      PARAMETER (VT1_SYM_HT      = 'b'           )! HT (HORIZONTAL TAB)
      PARAMETER (VT1_SYM_FF      = 'c'           )! FF (FORM FEED)
      PARAMETER (VT1_SYM_CR      = 'd'           )! CR (CARRIAGE RETURN)
      PARAMETER (VT1_SYM_LF      = 'e'           )! LF (LINE FEED)
      PARAMETER (VT1_SYM_DEGREE  = 'f'           )! DEGREE SYMBOL
      PARAMETER (VT1_SYM_PL_MIN  = 'g'           )! PLUS/MINUS +/-
      PARAMETER (VT1_SYM_NL      = 'h'           )! NL (NEW LINE)
      PARAMETER (VT1_SYM_VT      = 'i'           )! VT (VERTICAL TAB)
      PARAMETER (VT1_SYM_LRC     = 'j'           )! LOWER-RIGHT CORNER
      PARAMETER (VT1_SYM_URC     = 'k'           )! UPPER-RIGHT CORNER
      PARAMETER (VT1_SYM_ULC     = 'l'           )! UPPER-LEFT CORNER
      PARAMETER (VT1_SYM_LLC     = 'm'           )! LOWER-LEFT CORNER
      PARAMETER (VT1_SYM_CROSS   = 'n'           )! CROSSING LINES
      PARAMETER (VT1_SYM_SCAN1   = 'o'           )! HORIZONTAL LINE - SCAN 1
      PARAMETER (VT1_SYM_SCAN3   = 'p'           )! HORIZONTAL LINE - SCAN 3
      PARAMETER (VT1_SYM_SCAN5   = 'q'           )! HORIZONTAL LINE - SCAN 5
      PARAMETER (VT1_SYM_SCAN7   = 'r'           )! HORIZONTAL LINE - SCAN 7
      PARAMETER (VT1_SYM_SCAN9   = 's'           )! HORIZONTAL LINE - SCAN 9
      PARAMETER (VT1_SYM_LEFT_T  = 't'           )! LEFT "T"
      PARAMETER (VT1_SYM_RIGHT_T = 'u'           )! RIGHT "T"
      PARAMETER (VT1_SYM_LOWER_T = 'v'           )! BOTTOM "T"
      PARAMETER (VT1_SYM_UPPER_T = 'w'           )! TOP "T"
      PARAMETER (VT1_SYM_V_BAR   = 'x'           )! VERTICAL BAR
      PARAMETER (VT1_SYM_LE      = 'y'           )! LESS THAN OR EQUAL TO (.LE.)
      PARAMETER (VT1_SYM_GE      = 'z'           )! GREATER THAN OR EQUAL TO (.GE.)
      PARAMETER (VT1_SYM_PI      = '{'           )! PI (MATHEMATICAL)
      PARAMETER (VT1_SYM_NE      = '|'           )! NOT EQUAL TO (.NE.)
      PARAMETER (VT1_SYM_POUND   = '}'           )! UK POUND SIGN
      PARAMETER (VT1_SYM_DOT     = '~'           )! CENTERED DOT
C----------------------------------------------------------------------*
C                                                                      *
C     SCROLLING REGION COMMANDS                                        *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_SET_REGION  = 'r'           )! SCROLLING REGION PREFIX
      PARAMETER (VT1_RST_REGION  = ANS//';24r'   )! RESET SCROLLING REGION
C----------------------------------------------------------------------*
C                                                                      *
C     TAB STOP COMMANDS                                                *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_SET_TAB     = ESC//'H'      )! SET TAB AT CURRENT COLUMN
      PARAMETER (VT1_CLEAR_TAB   = ANS//'g'      )! CLEAR TAB AT CURRENT COLUMN
      PARAMETER (VT1_CLEAR_TABS  = ANS//'3g'     )! CLEAR ALL TABS
C----------------------------------------------------------------------*
C                                                                      *
C     MODE CONVERSION COMMANDS                                         *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (SET             = 'h'           )! SET MODE
      PARAMETER (RST             = 'l'           )! RESET MODE
      PARAMETER (VT1_NL_MODE     = ANS//'20'//SET)! NEW LINE MODE
      PARAMETER (VT1_LF_MODE     = ANS//'20'//RST)! LINE FEED MODE
      PARAMETER (VT1_CURSOR_APP  = ANS//'?1'//SET)! CURSOR KEY APPLICATION
      PARAMETER (VT1_CURSOR_KEY  = ANS//'?1'//RST)! CURSOR KEY CURSOR
      PARAMETER (VT1_ANSI        = ESC//'<'      )! ANSI (VT100) MODE
      PARAMETER (VT1_VT52        = ANS//'?2'//RST)! VT52 MODE
      PARAMETER (VT1_132_COLUMN  = ANS//'?3'//SET)! 132 COLUMN MODE
      PARAMETER (VT1_80_COLUMN   = ANS//'?3'//RST)! 80 COLUMN MODE
      PARAMETER (VT1_SMOOTH      = ANS//'?4'//SET)! SMOOTH SCROLLING
      PARAMETER (VT1_JUMP        = ANS//'?4'//RST)! JUMP SCROLLING
      PARAMETER (VT1_PAGE_LIGHT  = ANS//'?5'//SET)! REVERSE SCREEN MODE
      PARAMETER (VT1_PAGE_DARK   = ANS//'?5'//RST)! NORMAL SCREEN MODE
      PARAMETER (VT1_RELATIVE    = ANS//'?6'//SET)! RELATIVE ORIGIN MODE
      PARAMETER (VT1_ABSOLUTE    = ANS//'?6'//RST)! ABSOLUTE ORIGIN MODE
      PARAMETER (VT1_WRAP_ON     = ANS//'?7'//SET)! WRAPAROUND ON
      PARAMETER (VT1_WRAP_OFF    = ANS//'?7'//RST)! WRAPAROUND OFF
      PARAMETER (VT1_REPEAT_ON   = ANS//'?8'//SET)! AUTO REPEAT ON
      PARAMETER (VT1_REPEAT_OFF  = ANS//'?8'//RST)! AUTO REPEAT OFF
      PARAMETER (VT1_RETRACE_ON  = ANS//'?9'//SET)! INTERLACE ON
      PARAMETER (VT1_RETRACE_OFF = ANS//'?9'//RST)! INTERLACE OFF
      PARAMETER (VT1_GRAPHIC_ON  = ESC//'1'      )! GRAPHIC PROC. ON
      PARAMETER (VT1_GRAPHIC_OFF = ESC//'2'      )! GRAPHIC PROC. OFF
      PARAMETER (VT1_KEYPAD_APP  = ESC//'='      )! KEYPAD APPLICATION MODE
      PARAMETER (VT1_KEYPAD_NUM  = ESC//'>'      )! KEYPAD NUMERIC MODE
C----------------------------------------------------------------------*
C                                                                      *
C     DEVICE STATUS REPORT COMMANDS                                    *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_POSITION    = ANS//'6n'     )! CURSOR POSITION REPORT
      PARAMETER (VT1_POS_REPORT  = 'R'           )! POSITION REPORT SUFFIX
      PARAMETER (VT1_STATUS      = ANS//'5n'     )! DEVICE STATUS REPORT
      PARAMETER (VT1_ATTRIBUTES  = ANS//'c'      )! DEVICE ATTRIBUTES REPORT
      PARAMETER (VT1_REQUEST     = 'x'           )! REQUEST VT100 PARAMETERS
      PARAMETER (VT1_ANSWERBACK  = ENQ           )! ANSWERBACK ENQUIRY
C----------------------------------------------------------------------*
C                                                                      *
C     POWER-UP RESET COMMAND                                           *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_RESET       = ESC//'c'      )! RESET VT100 TERMINAL
C----------------------------------------------------------------------*
C                                                                      *
C     TERMINAL TEST COMMANDS                                           *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_ALIGNMENT   = '#8'          )! SCREEN ALIGNMENT TEST
      PARAMETER (VT1_TEST        = ANS//'2'//DLM )! CONFIDENCE TESTS
      PARAMETER (VT1_TEST_ENDFIX = 'y'           )! TEST POSTFIX
C----------------------------------------------------------------------*
C                                                                      *
C     AUXILIARY I/O PORT COMMANDS                                      *
C                                                                      *
C----------------------------------------------------------------------*
      PARAMETER (VT1_PRINT_PAGE  = ANS//'0'//'i' )! PRINT ENTIRE CRT SCREEN
      PARAMETER (VT1_PRINTER_OFF = ANS//'4'//'i' )! TURN I/O PORT OFF (EXIT)
      PARAMETER (VT1_PRINTER_ON  = ANS//'5'//'i' )! TURN I/O PORT ON (ENTER)

C----------------------------------------------------------------------*
C                                                                      *
C     EXTERNAL                                                         *
C                                                                      *
C----------------------------------------------------------------------*
C     EXTERNAL SYSTEM SERVICE AND QIO FUNCTION CODES                   *
C----------------------------------------------------------------------*
      EXTERNAL IO$_READPROMPT
      EXTERNAL IO$_READVBLK
      EXTERNAL IO$_SENSECHAR
      EXTERNAL IO$_SENSEMODE
      EXTERNAL IO$_SETCHAR
      EXTERNAL IO$_SETMODE
      EXTERNAL IO$_WRITEVBLK
      EXTERNAL IO$M_NOFORMAT
      EXTERNAL IO$M_TIMED
      EXTERNAL IO$M_TRMNOECHO
      EXTERNAL IO$M_TYPEAHDCNT
      EXTERNAL SS$_DEVCMDERR
      EXTERNAL SS$_ENDOFFILE
      EXTERNAL SS$_NORMAL
      EXTERNAL TT$M_ESCAPE

C----------------------------------------------------------------------*
C                                                                      *
C     EQUIVALENCE                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C     EQUIVALENCE LOGICAL ENABLE NAMES WITH LOCAL SYMBOLS              *
C----------------------------------------------------------------------*
C     THIS LIST OF EQUIVALENCES ENABLES THE PHYSICAL LOCATIONS IN      *
C     THE DEVICE ENABLE TABLE TO BE REFERENCED BY LOGICAL NAMES, THUS  *
C     ELIMINATING THE HARD ASSOCIATION OF LOGICAL NAMES AND PHYSICAL   *
C     FUNCTION ENABLES.                                                *
C----------------------------------------------------------------------*
      EQUIVALENCE (DEVICE_TABLE(1),HAZELTINE)
      EQUIVALENCE (DEVICE_TABLE(2),HP264X)
      EQUIVALENCE (DEVICE_TABLE(3),ISC_8001)
      EQUIVALENCE (DEVICE_TABLE(4),LA36)
      EQUIVALENCE (DEVICE_TABLE(5),LA120)
      EQUIVALENCE (DEVICE_TABLE(6),PRINTRONIX)
      EQUIVALENCE (DEVICE_TABLE(7),QUME)
      EQUIVALENCE (DEVICE_TABLE(8),VT52)
      EQUIVALENCE (DEVICE_TABLE(9),VT100)

C----------------------------------------------------------------------*
C                                                                      *
C     CHARACTER                                                        *
C                                                                      *
C----------------------------------------------------------------------*
      CHARACTER*(NAM) DEVICE_IDNAM   (MAXCRT)
      CHARACTER*( * ) INPUT_STRING
      CHARACTER*( * ) OUTPUT_STRING
      CHARACTER*(SIX) PARM1
      CHARACTER*(SIX) PARM2
      CHARACTER*(ONE) POINT
      CHARACTER*(SIZ) STRING
      CHARACTER*(NAM) STRING1
      CHARACTER*(NAM) STRING2

C----------------------------------------------------------------------*
C                                                                      *
C     INTEGER*2                                                        *
C                                                                      *
C----------------------------------------------------------------------*
      INTEGER*2 IO_MODE        (FOUR)
      INTEGER*2 IO_STATUS      (FOUR)
      INTEGER*2 QIO_MODE_BUF   (FOUR)
      INTEGER*2 QIO_STATUS_BUF (FOUR)
C----------------------------------------------------------------------*
C                                                                      *
C     INTEGER*4                                                        *
C                                                                      *
C----------------------------------------------------------------------*
      INTEGER*4 ASCII_CODE
      INTEGER*4 BOTTOM_LINE
      INTEGER*4 BYTCNT
      INTEGER*4 BYTONE
      INTEGER*4 CLASS
      INTEGER*4 COL
      INTEGER*4 COLOR
      INTEGER*4 COLOR_MASK
      INTEGER*4 COUNT
      INTEGER*4 COUNTER
      INTEGER*4 CRT_DRIVER
      INTEGER*4 CRT_QIO_READ
      INTEGER*4 CRT_QIO_WRITE
      INTEGER*4 CRT_READER_CHAN
      INTEGER*4 CRT_WRITER_CHAN
      INTEGER*4 DEVICE_TYPE
      INTEGER*4 DEVICE_TYPE_IO
      INTEGER*4 FIRST
      INTEGER*4 IO_FUNCTION
      INTEGER*4 IO$_READPROMPT
      INTEGER*4 IO$_READVBLK
      INTEGER*4 IO$_SENSECHAR
      INTEGER*4 IO$_SENSEMODE
      INTEGER*4 IO$_SETCHAR
      INTEGER*4 IO$_SETMODE
      INTEGER*4 IO$_WRITEVBLK
      INTEGER*4 IO$M_NOFORMAT
      INTEGER*4 IO$M_TIMED
      INTEGER*4 IO$M_TRMNOECHO
      INTEGER*4 IO$M_TYPEAHDCNT
      INTEGER*4 LAST_COLUMN
      INTEGER*4 LAST
      INTEGER*4 LED_NUMBER
      INTEGER*4 LENGTH
      INTEGER*4 LIB$PUT_OUTPUT
      INTEGER*4 LIMIT
      INTEGER*4 LIMIT_ONE
      INTEGER*4 LIMIT_TWO
      INTEGER*4 PAGLEN
      INTEGER*4 PAGWID
      INTEGER*4 PARSE
      INTEGER*4 PITCH
      INTEGER*4 POINTER
      INTEGER*4 REPEAT_COUNT
      INTEGER*4 REQUEST_PARAM
      INTEGER*4 ROW
      INTEGER*4 SS$_DEVCMDERR
      INTEGER*4 SS$_ENDOFFILE
      INTEGER*4 SS$_NORMAL
      INTEGER*4 STATUS
      INTEGER*4 SYS$ASSIGN
      INTEGER*4 SYS$GETMSG
      INTEGER*4 SYS$QIO
      INTEGER*4 SYS$QIOW
      INTEGER*4 SYS$TRNLOG
      INTEGER*4 SWITCH_LINES
      INTEGER*4 TERMCHAR
      INTEGER*4 TEST
      INTEGER*4 TIME_PERIOD
      INTEGER*4 TOP_LINE
      INTEGER*4 TRMCHR
      INTEGER*4 TT$M_ESCAPE
      INTEGER*4 TYPE
      INTEGER*4 X
      INTEGER*4 X_COL
      INTEGER*4 X_LIMIT
      INTEGER*4 X_MAX
      INTEGER*4 X_ORG
      INTEGER*4 X_POINT
      INTEGER*4 X_START
      INTEGER*4 X_STOP
      INTEGER*4 Y
      INTEGER*4 Y_ROW
      INTEGER*4 Y_LIMIT
      INTEGER*4 Y_MAX
      INTEGER*4 Y_ORG
      INTEGER*4 Y_POINT
      INTEGER*4 Y_START
      INTEGER*4 Y_STOP
      INTEGER*4 YSCAN

C----------------------------------------------------------------------*
C                                                                      *
C     LOGICAL*4                                                        *
C                                                                      *
C----------------------------------------------------------------------*
      LOGICAL*4 DEVICE_AUXIO   (MAXCRT)
      LOGICAL*4 DEVICE_TABLE   (MAXCRT)
      LOGICAL*4 ERROR
      LOGICAL*4 FLAG
      LOGICAL*4 HAZELTINE
      LOGICAL*4 HP264X
      LOGICAL*4 ISC_8001
      LOGICAL*4 LA36
      LOGICAL*4 LA120
      LOGICAL*4 NODATA
      LOGICAL*4 PRINTRONIX
      LOGICAL*4 QUME
      LOGICAL*4 UNKNOWN
      LOGICAL*4 VT52
      LOGICAL*4 VT100

      INCLUDE 'CRT.CMM'

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C           D E V I C E    N A M E    D A T A    T A B L E             *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
      DATA DEVICE_IDNAM     /  'HAZELTINE      ',  'HP264X         ',
     1                         'ISC_8001       ',  'LA36           ',
     2                         'LA120          ',  'PRINTRONIX     ',
     3                         'QUME           ',  'VT52           ',
     4                         'VT100          ',  '               '   /

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C     DRIVER INITIALIZATION ENTRYPOINT                                 *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     INITIALIZE CHARACTER I/O INTERFACE                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_INITIALIZE

      CRT_QIO_READ =(%LOC(IO$_READVBLK))
      CRT_QIO_WRITE=(%LOC(IO$_WRITEVBLK) .OR. %LOC(IO$M_NOFORMAT))
C----------------------------------------------------------------------*
C                                                                      *
C     TRANSLATE LOGICAL NAME TO PHYSICAL DEVICE                        *
C                                                                      *
C----------------------------------------------------------------------*
      STATUS=SYS$TRNLOG(CRT_READER,,STRING1,,,)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700

      STATUS=SYS$TRNLOG(CRT_WRITER,,STRING2,,,)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700
C----------------------------------------------------------------------*
C                                                                      *
C     ASSIGN COMMAND INPUT AND DISPLAY OUTPUT CHANNELS                 *
C                                                                      *
C----------------------------------------------------------------------*
      CALL DELETE_CONTROL(STRING1)
      STATUS=SYS$ASSIGN(STRING1(ONE:LENGTH(STRING1)),CRT_READER_CHAN,,)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700

      CALL DELETE_CONTROL(STRING2)
      STATUS=SYS$ASSIGN(STRING2(ONE:LENGTH(STRING2)),CRT_WRITER_CHAN,,)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700
C----------------------------------------------------------------------*
C                                                                      *
C     DETERMINE CURRENT PROCESS TERMINAL MODE CHARACTERISTICS          *
C                                                                      *
C----------------------------------------------------------------------*
      STATUS=SYS$QIOW(,%VAL(CRT_READER_CHAN),
     1     %VAL(%LOC(IO$_SENSEMODE)),
     2     IO_STATUS,,,
     3     IO_MODE,,,,,)

      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700
      CALL MOVC3(QUAD,%LOC(IO_MODE),%LOC(QIO_MODE_BUF))

C----------------------------------------------------------------------*
C                                                                      *
C     DETERMINE SPECIFIC TERMINAL TYPE (MOSTEK LOCAL DEFINITION)       *
C                                                                      *
C----------------------------------------------------------------------*
C     FIRST INITIALIZE THE CURRENT TERMINAL TYPE                       *
C----------------------------------------------------------------------*
      DO POINTER=ONE,MAXCRT
           DEVICE_TABLE(POINTER)=.FALSE.
      ENDDO
C----------------------------------------------------------------------*
C     TRANSLATE THE CURRENT PROCESS SYMBOL DEFINING THE TERMINAL TYPE  *
C----------------------------------------------------------------------*
      STATUS=SYS$TRNLOG(CRT_TERMTYPE,,STRING,,,)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700

      CALL DELETE_CONTROL(STRING)
      LIMIT=INDEX(STRING,PERIOD)-ONE
      IF (LIMIT .LE. ZERO) LIMIT=LENGTH(STRING)
      DEVICE_TYPE=PARSE(DEVICE_IDNAM,ONE,MAXCRT,STRING(ONE:LIMIT))
      IF (DEVICE_TYPE .NE. ZERO) DEVICE_TABLE(DEVICE_TYPE)=.TRUE.

      STRING=STRING((LIMIT+TWO):)
      LIMIT=LENGTH(STRING)
      DEVICE_TYPE_IO=PARSE(DEVICE_IDNAM,ONE,MAXCRT,STRING(ONE:LIMIT))
      IF (DEVICE_TYPE_IO .NE. ZERO) DEVICE_AUXIO(DEVICE_TYPE_IO)=.TRUE.

C----------------------------------------------------------------------*
C                                                                      *
C     INITIALIZE THE TERMINAL TO DEFAULT MODE SETTINGS                 *
C                                                                      *
C----------------------------------------------------------------------*
      REPEAT_COUNT=ONE


      IF (VT100) THEN

           CALL CRT_CMD(VT1_ANSI)
           LAST_COLUMN=VT1_COLUMN_MAX

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C     DRIVER CONTROL AND STATUS DISPLAY ENTRYPOINTS                    *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     RETURN QUADWORD I/O STATUS BLOCK FROM LAST I/O OPERATION         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_IO_STATUS(QIO_STATUS_BUF)





      DO COUNTER=ONE,FOUR
           QIO_STATUS_BUF(COUNTER)=IO_STATUS(COUNTER)
      ENDDO





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SENSE CURRENT MODE OF TERMINAL CONFIGURATION                     *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT RETURNS THE CURRENT STATUS OF THE TERMINAL       *
C     CHARACTERISTICS WHICH ARE PROCESS RELATED IN NATURE.             *
C----------------------------------------------------------------------*
      ENTRY CRT_SENSE_MODE(CLASS,TYPE,PAGWID,PAGLEN,TRMCHR)





C----------------------------------------------------------------------*
C     SENSE TERMINAL MODE                                              *
C----------------------------------------------------------------------*
      STATUS=SYS$QIOW(,%VAL(CRT_READER_CHAN),
     1     %VAL(%LOC(IO$_SENSEMODE)),
     2     IO_STATUS,,,
     3     QIO_MODE_BUF,,,,,)

      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700
C----------------------------------------------------------------------*
C     LOAD CLASS, TYPE, PAGE WIDTH, PAGE LENGTH, AND CHARACTERISTICS   *
C----------------------------------------------------------------------*
      CALL MOVC5(BYTE,(%LOC(QIO_MODE_BUF)+0),ZERO,LONG,%LOC(CLASS))
      CALL MOVC5(BYTE,(%LOC(QIO_MODE_BUF)+1),ZERO,LONG,%LOC(TYPE))
      CALL MOVC5(WORD,(%LOC(QIO_MODE_BUF)+2),ZERO,LONG,%LOC(PAGWID))
      CALL MOVC5(BYT3,(%LOC(QIO_MODE_BUF)+4),ZERO,LONG,%LOC(TRMCHR))
      CALL MOVC5(BYTE,(%LOC(QIO_MODE_BUF)+7),ZERO,LONG,%LOC(PAGLEN))




      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SENSE MODE OF CURRENT TYPEAHEAD BUFFER                           *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT SENSES THE CURRENT STATUS OF THE TYPEAHEAD       *
C     BUFFER, RETURNING THE NUMBER OF CHARACTERS IN THE BUFFER AND     *
C     THE FIRST CHARACTER IN THE BUFFER.                               *
C----------------------------------------------------------------------*
      ENTRY CRT_TYPEAHEAD(BYTCNT,BYTONE)





C----------------------------------------------------------------------*
C     SENSE TERMINAL TYPEAHEAD BUFFER                                  *
C----------------------------------------------------------------------*
      STATUS=SYS$QIOW(,%VAL(CRT_READER_CHAN),
     1     %VAL(%LOC(IO$_SENSEMODE) .OR. %LOC(IO$M_TYPEAHDCNT)),
     2     IO_STATUS,,,
     3     QIO_STATUS_BUF,,,,,)

      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700
C----------------------------------------------------------------------*
C     SET CURRENT TYPEAHEAD COUNT AND FIRST CHARACTER IN BUFFER        *
C----------------------------------------------------------------------*
      CALL MOVC5(WORD,(%LOC(QIO_STATUS_BUF)+0),ZERO,LONG,%LOC(BYTCNT))
      CALL MOVC5(BYTE,(%LOC(QIO_STATUS_BUF)+2),ZERO,LONG,%LOC(BYTONE))





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET CURRENT MODE OF TERMINAL CONFIGURATION                       *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT ENABLES THE USER TO SPECIFY THE CURRENT STATUS   *
C     OF THE TERMINAL CHARACTERISTICS WHICH ARE CRT PROCESS RELATED    *
C----------------------------------------------------------------------*
      ENTRY CRT_SET_MODE(CLASS,TYPE,PAGWID,PAGLEN,TRMCHR)




C----------------------------------------------------------------------*
C     LOAD CLASS, TYPE, PAGE WIDTH, PAGE LENGTH, AND CHARACTERISTICS   *
C----------------------------------------------------------------------*
      CALL MOVC3(BYTE,%LOC(CLASS),(%LOC(QIO_MODE_BUF)+0))
      CALL MOVC3(BYTE,%LOC(TYPE),(%LOC(QIO_MODE_BUF)+1))
      CALL MOVC3(WORD,%LOC(PAGWID),(%LOC(QIO_MODE_BUF)+2))
      CALL MOVC3(BYTE,%LOC(PAGLEN),(%LOC(QIO_MODE_BUF)+7))
      CALL MOVC3(BYT3,%LOC(TRMCHR),(%LOC(QIO_MODE_BUF)+4))





      GO TO 800

C----------------------------------------------------------------------*
C                                                                      *
C     ENABLE ESCAPE SEQUENCE ENTRY AND VALIDATION                      *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT ENABLES TERMINAL ESCAPE SEQUENCES TO BE ENTERED  *
C     BY THE USER, THUS ENABLING THE ALTERNATE KEYPAD AND CURSOR       *
C     APPLICATION KEYPAD MODES OF THE VT100 TO BE USED.                *
C----------------------------------------------------------------------*
      ENTRY CRT_ESCAPE_ON





      CALL MOVC5(BYT3,(%LOC(QIO_MODE_BUF)+4),ZERO,LONG,%LOC(TERMCHAR))
      TERMCHAR=IOR(TERMCHAR,%LOC(TT$M_ESCAPE))
      CALL MOVC5(BYT3,%LOC(TERMCHAR),ZERO,BYT3,(%LOC(QIO_MODE_BUF)+4))





      GO TO 800

C----------------------------------------------------------------------*
C                                                                      *
C     DISABLE ESCAPE SEQUENCE ENTRY AND VALIDATION                     *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT DISABLES ESCAPE SEQUENCE ENTRY AND VALIDATION    *
C     FOR THE GIVEN TERMINAL, MAKING THE CURSOR KEYS AND NUMERIC       *
C     KEYPAD APPLICABLE FOR NON-ESCAPE ENTRY FUNCTIONS.                *
C----------------------------------------------------------------------*
      ENTRY CRT_ESCAPE_OFF





      CALL MOVC5(BYT3,(%LOC(QIO_MODE_BUF)+4),ZERO,LONG,%LOC(TERMCHAR))
      TERMCHAR=IAND(TERMCHAR,NOT(%LOC(TT$M_ESCAPE)))
      CALL MOVC5(BYT3,%LOC(TERMCHAR),ZERO,BYT3,(%LOC(QIO_MODE_BUF)+4))





      GO TO 800

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C     DRIVER EXIT AND CLEANUP ENTRYPOINT                               *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     RESTORE THE CRT TO ITS ORIGINAL CONDITION (MOSTEK STANDARD)      *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_EXIT





C----------------------------------------------------------------------*
C                                                                      *
C     RESTORE PROCESS TERMINAL MODE CHARACTERISTICS                    *
C                                                                      *
C----------------------------------------------------------------------*
      STATUS=SYS$QIOW(,%VAL(CRT_READER_CHAN),
     1     %VAL(%LOC(IO$_SETMODE)),
     2     IO_STATUS,,,
     3     IO_MODE,,,,,)
C----------------------------------------------------------------------*
C                                                                      *
C     RESTORE SOFTWARE CONDITION OF HARDWARE TERMINAL                  *
C                                                                      *
C----------------------------------------------------------------------*
      IF (VT100) THEN

           CALL CRT_CMD(VT1_ANSI)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(
     1          ISC_PLOT_ESCAPE//
     2          ISC_FOREGROUND//
     3          ISC_BLINKA7_OFF)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     REPETITION COUNT COMMAND                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_REPEAT(COUNT)





      REPEAT_COUNT=COUNT





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     CURSOR MOVEMENT COMMANDS                                         *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     HOME CURSOR UP (UPPER LEFT CORNER)                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HOME
      ENTRY CRT_HOME_UP





      IF (VT100) THEN

           CALL CRT_CMD(VT1_HOME_UP)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(VT5_HOME_UP)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HOME_UP)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     HOME CURSOR UP (UPPER RIGHT CORNER)                              *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HOME_RIGHT





      IF (VT100) THEN

           CALL CRT_CMD(VT1_HOME_RIGHT)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(VT5_HOME_RIGHT)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HOME_RIGHT)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     HOME CURSOR DOWN (LOWER LEFT CORNER)                             *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HOME_DOWN





      IF (VT100) THEN

           CALL CRT_CMD(VT1_HOME_DOWN)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(VT5_HOME_DOWN)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HOME_DOWN)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     HOME CURSOR DOWN (LOWER RIGHT CORNER)                            *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HOME_BOTTOM





      IF (VT100) THEN

           CALL CRT_CMD(VT1_HOME_BOTTOM)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(VT5_HOME_BOTTOM)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HOME_BOTTOM)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CURSOR UP                                                        *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_MOVE_UP(COUNT)





      IF (VT100) THEN

           CALL INTFMT(COUNT,PARM1,LIMIT)
           CALL CRT_CMD(ANS//PARM1(ONE:LIMIT)//VT1_MOVE_UP)

      ELSE IF (VT52) THEN

           REPEAT_COUNT=COUNT
           CALL CRT_CMD(VT5_MOVE_UP)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_MOVE_UP)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CURSOR DOWN                                                      *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_MOVE_DOWN(COUNT)





      IF (VT100) THEN

           CALL INTFMT(COUNT,PARM1,LIMIT)
           CALL CRT_CMD(ANS//PARM1(ONE:LIMIT)//VT1_MOVE_DOWN)

      ELSE IF (VT52) THEN

           REPEAT_COUNT=COUNT
           CALL CRT_CMD(VT5_MOVE_DOWN)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CURSOR FORWARD (RIGHT)                                           *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_MOVE_RIGHT(COUNT)





      IF (VT100) THEN

           CALL INTFMT(COUNT,PARM1,LIMIT)
           CALL CRT_CMD(ANS//PARM1(ONE:LIMIT)//VT1_MOVE_RIGHT)

      ELSE IF (VT52) THEN

           REPEAT_COUNT=COUNT
           CALL CRT_CMD(VT5_MOVE_RIGHT)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_MOVE_RIGHT)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CURSOR BACKWARD (LEFT)                                           *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_MOVE_LEFT(COUNT)





      IF (VT100) THEN

           CALL INTFMT(COUNT,PARM1,LIMIT)
           CALL CRT_CMD(ANS//PARM1(ONE:LIMIT)//VT1_MOVE_LEFT)

      ELSE IF (VT52) THEN

           REPEAT_COUNT=COUNT
           CALL CRT_CMD(VT5_MOVE_LEFT)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_MOVE_LEFT)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     DIRECT CURSOR ADDRESSING                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_MOVE(ROW,COL)





      IF (VT100) THEN

           CALL INTFMT(ROW,PARM1,LIMIT_ONE)
           CALL INTFMT(COL,PARM2,LIMIT_TWO)
           CALL CRT_CMD(ANS//PARM1(ONE:LIMIT_ONE)//
     1          DLM//PARM2(ONE:LIMIT_TWO)//VT1_MOVE)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(ESC//
     1          CHAR(VT5_ROW_ORG+ROW)//CHAR(VT5_COLUMN_ORG+COL))

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           X_COL=ISC_COLUMN_MAX-COL
           Y_ROW=ISC_ROW_MAX-ROW
           CALL CRT_CMD(ISC_MOVE//CHAR(X_COL)//CHAR(Y_ROW))

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     INDEX                                                            *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_INDEX
      ENTRY CRT_INDEX_DOWN





      IF (VT100) THEN

           CALL CRT_CMD(VT1_INDEX_DOWN)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(VT5_INDEX_DOWN)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     INDEX TO NEXT LINE                                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_INDEX_NEXT





      IF (VT100) THEN

           CALL CRT_CMD(VT1_INDEX_NEXT)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(CRLF)

      ELSE IF (HAZELTINE) THEN

           CALL CRT_CMD(CRLF)

      ELSE IF (HP264X) THEN

           CALL CRT_CMD(CRLF)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(CRLF)

      ELSE IF (LA36) THEN

           CALL CRT_CMD(CRLF)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(CRLF)

      ELSE IF (PRINTRONIX) THEN

           CALL CRT_CMD(CRLF)

      ELSE IF (QUME) THEN

           CALL CRT_CMD(CRLF)

      ELSE IF (UNKNOWN) THEN

           CALL CRT_CMD(CRLF)

      ENDIF





      GO TO 900
C----------------------------------------------------------------------*
C                                                                      *
C     REVERSE INDEX                                                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_INDEX_UP





      IF (VT100) THEN

           CALL CRT_CMD(VT1_INDEX_UP)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(LF)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SAVE CURSOR AND ATTRIBUTES                                       *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_STORE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_STORE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     RESTORE CURSOR AND ATTRIBUTES                                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_RESTORE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_RESTORE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     HORIZONTAL AND VERTICAL LINE SIZE COMMANDS                       *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SET HORIZONTAL PITCH (CHARACTERS/INCH)                           *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HORIZONTAL(PITCH)





      IF (VT100) THEN

           IF (PITCH .LE. 12) THEN
                CALL CRT_CMD(VT1_80_COLUMN)
           ELSE
                CALL CRT_CMD(VT1_132_COLUMN)
           ENDIF

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           IF (PITCH .LE. 05) THEN
                CALL CRT_CMD(LA1_H_PITCH_05)
           ELSE IF (PITCH .LE. 06) THEN
                CALL CRT_CMD(LA1_H_PITCH_06)
           ELSE IF (PITCH .LE. 07) THEN
                CALL CRT_CMD(LA1_H_PITCH_07)
           ELSE IF (PITCH .LE. 08) THEN
                CALL CRT_CMD(LA1_H_PITCH_08)
           ELSE IF (PITCH .LE. 10) THEN
                CALL CRT_CMD(LA1_H_PITCH_10)
           ELSE IF (PITCH .LE. 12) THEN
                CALL CRT_CMD(LA1_H_PITCH_12)
           ELSE IF (PITCH .LE. 13) THEN
                CALL CRT_CMD(LA1_H_PITCH_13)
           ELSE
                CALL CRT_CMD(LA1_H_PITCH_16)
           ENDIF

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET VERTICAL PITCH (LINES/INCH)                                  *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_VERTICAL(PITCH)





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           IF (PITCH .LE. 02) THEN
                CALL CRT_CMD(LA1_V_PITCH_02)
           ELSE IF (PITCH .LE. 03) THEN
                CALL CRT_CMD(LA1_V_PITCH_03)
           ELSE IF (PITCH .LE. 04) THEN
                CALL CRT_CMD(LA1_V_PITCH_04)
           ELSE IF (PITCH .LE. 06) THEN
                CALL CRT_CMD(LA1_V_PITCH_06)
           ELSE IF (PITCH .LE. 08) THEN
                CALL CRT_CMD(LA1_V_PITCH_08)
           ELSE
                CALL CRT_CMD(LA1_V_PITCH_12)
           ENDIF

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CHANGE THIS LINE TO DOUBLE-HEIGHT TOP HALF                       *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_DOUBLE_TOP





      IF (VT100) THEN

           CALL CRT_CMD(VT1_DOUBLE_TOP)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CHANGE THIS LINE TO DOUBLE-HEIGHT BOTTOM HALF                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_DOUBLE_BOT





      IF (VT100) THEN

           CALL CRT_CMD(VT1_DOUBLE_BOT)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CHANGE THIS LINE TO SINGLE-WIDTH SINGLE-HEIGHT                   *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SINGLE_WIDE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SINGLE_WIDE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CHANGE THIS LINE TO DOUBLE-WIDTH SINGLE-HEIGHT                   *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_DOUBLE_WIDE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_DOUBLE_WIDE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     CHARACTER ATTRIBUTE COMMANDS                                     *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     ALL ATTRIBUTES OFF                                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_TURN_OFF





      IF (VT100) THEN

           CALL CRT_CMD(VT1_TURN_OFF)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_FOREGROUND//ISC_BLINKA7_OFF)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     BOLD ON                                                          *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_BOLD





      IF (VT100) THEN

           CALL CRT_CMD(VT1_BOLD)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     UNDERSCORE ON                                                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_UNDER





      IF (VT100) THEN

           CALL CRT_CMD(VT1_UNDER)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     BLINK ON                                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_BLINK





      IF (VT100) THEN

           CALL CRT_CMD(VT1_BLINK)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_BLINK)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     REVERSE VIDEO ON                                                 *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_REVERSE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_REVERSE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_BACKGROUND)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     FOREGROUND/BACKGROUND COLOR SELECTION                            *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SELECT USER-PASSED COLOR                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_COLOR(COLOR_MASK)
      ENTRY CRT_HUE(COLOR_MASK)





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           COLOR=ISC_COLOR_BIAS+(COLOR_MASK .AND. ISC_COLOR_MASK)
           CALL CRT_CMD(CHAR(COLOR))

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     FOREGROUND/BACKGROUND COLOR SELECTION                            *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SELECT COLOR BLACK                                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HUE_BLACK





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HUE_BLACK)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     FOREGROUND/BACKGROUND COLOR SELECTION                            *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SELECT COLOR RED                                                 *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HUE_RED





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HUE_RED)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     FOREGROUND/BACKGROUND COLOR SELECTION                            *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SELECT COLOR GREEN                                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HUE_GREEN





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HUE_GREEN)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     FOREGROUND/BACKGROUND COLOR SELECTION                            *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SELECT COLOR YELLOW                                              *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HUE_YELLOW





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HUE_YELLOW)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     FOREGROUND/BACKGROUND COLOR SELECTION                            *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SELECT COLOR BLUE                                                *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HUE_BLUE





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HUE_BLUE)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     FOREGROUND/BACKGROUND COLOR SELECTION                            *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SELECT COLOR MAGENTA                                             *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HUE_MAGENTA





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HUE_MAGENTA)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     FOREGROUND/BACKGROUND COLOR SELECTION                            *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SELECT COLOR CYAN                                                *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HUE_CYAN





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HUE_CYAN)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     FOREGROUND/BACKGROUND COLOR SELECTION                            *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SELECT COLOR WHITE                                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_HUE_WHITE





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_HUE_WHITE)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     ERASING COMMANDS                                                 *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     ERASE FROM CURSOR TO END OF LINE                                 *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_CLEAR_EOL





      IF (VT100) THEN

           CALL CRT_CMD(VT1_CLEAR_EOL)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(VT5_CLEAR_EOL)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     ERASE FROM BEGINNING OF LINE TO CURSOR                           *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_CLEAR_BOL





      IF (VT100) THEN

           CALL CRT_CMD(VT1_CLEAR_BOL)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     ERASE ENTIRE LINE CONTAINING CURSOR                              *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_CLEAR_LINE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_CLEAR_LINE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_CLEAR_LINE)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     ERASE FROM CURSOR TO END OF SCREEN                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_CLEAR_EOP





      IF (VT100) THEN

           CALL CRT_CMD(VT1_CLEAR_EOP)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(VT5_CLEAR_EOP)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     ERASE FROM BEGINNING OF SCREEN TO CURSOR                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_CLEAR_BOP





      IF (VT100) THEN

           CALL CRT_CMD(VT1_CLEAR_BOP)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     ERASE ENTIRE SCREEN                                              *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_CLEAR_PAGE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_CLEAR_PAGE)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(VT5_HOME_UP//VT5_CLEAR_EOP)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_CLEAR_PAGE)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     PROGRAMMABLE LIGHT EMITTING DIODE (LED) COMMANDS                 *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     ALL LED DISPLAYS OFF                                             *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_CLEAR_LEDS





      IF (VT100) THEN

           CALL CRT_CMD(VT1_CLEAR_LEDS)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     TURN ON A SPECIFIC LED (0-4)                                     *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SET_LED(LED_NUMBER)





      IF (VT100) THEN

           CALL INTFMT(LED_NUMBER,PARM1,LIMIT)
           CALL CRT_CMD(ANS//PARM1(ONE:ONE)//VT1_SET_LED)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     CHARACTER SET COMMANDS                                           *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     UNITED KINGDOM (UK) G0 CHARACTER SET                             *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_UK
      ENTRY CRT_UK_G0





      IF (VT100) THEN

           CALL CRT_CMD(VT1_UK_G0)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_PLOT_ESCAPE)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_UK)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     UNITED KINGDOM (UK) G1 CHARACTER SET                             *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_UK_G1





      IF (VT100) THEN

           CALL CRT_CMD(VT1_UK_G1)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_PLOT_ESCAPE)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_UK)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     UNITED STATES (USASCII) G0 CHARACTER SET                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_USASCII
      ENTRY CRT_USASCII_G0





      IF (VT100) THEN

           CALL CRT_CMD(VT1_USASCII_G0)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_PLOT_ESCAPE)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_USASCII)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     UNITED STATES (USASCII) G1 CHARACTER SET                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_USASCII_G1





      IF (VT100) THEN

           CALL CRT_CMD(VT1_USASCII_G1)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_PLOT_ESCAPE)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_USASCII)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SPECIAL GRAPHICS CHARACTERS AND LINE DRAWING SET (G0)            *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_GRAPHICS
      ENTRY CRT_GRAPHICS_G0





      IF (VT100) THEN

           CALL CRT_CMD(VT1_GRAPHICS_G0)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_GRAPHICS)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_APL)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SPECIAL GRAPHICS CHARACTERS AND LINE DRAWING SET (G1)            *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_GRAPHICS_G1





      IF (VT100) THEN

           CALL CRT_CMD(VT1_GRAPHICS_G1)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           CALL CRT_CMD(ISC_GRAPHICS)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_FINLAND)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     ALTERNATE CHARACTER ROM (G0)                                     *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_ROM1
      ENTRY CRT_ROM1_G0





      IF (VT100) THEN

           CALL CRT_CMD(VT1_ROM1_G0)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_DENMARK)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     ALTERNATE CHARACTER ROM (G1)                                     *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_ROM1_G1





      IF (VT100) THEN

           CALL CRT_CMD(VT1_ROM1_G1)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_SWEDEN)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     ALTERNATE CHARACTER ROM - SPECIAL GRAPHICS CHARACTERS (G0)       *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_ROM2
      ENTRY CRT_ROM2_G0





      IF (VT100) THEN

           CALL CRT_CMD(VT1_ROM2_G0)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_GERMANY)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     ALTERNATE CHARACTER ROM - SPECIAL GRAPHICS CHARACTERS (G1)       *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_ROM2_G1





      IF (VT100) THEN

           CALL CRT_CMD(VT1_ROM2_G1)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_FRANCE)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     CHARACTER INPUT/OUTPUT COMMANDS                                  *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     OUTPUT ASCII CHARACTER CODE                                      *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_ASCII(ASCII_CODE)





      CALL CRT_CMD(CHAR(ASCII_CODE))





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     BELL CHARACTER                                                   *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_BELL





      CALL CRT_CMD(BELL)





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     TAB CHARACTER                                                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_TAB





      CALL CRT_CMD(TAB)





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     LINE FEED CHARACTER                                              *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_LINEFEED





      CALL CRT_CMD(LF)





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     VERTICAL TAB (VFU) CHARACTER                                     *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_VFU





      CALL CRT_CMD(VT)





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     FORM FEED CHARACTER                                              *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_FORMFEED





      CALL CRT_CMD(FORM_FEED)





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CARRIAGE RETURN CHARACTER                                        *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_RETURN





      CALL CRT_CMD(CR)





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CARRIAGE RETURN AND LINE FEED CHARACTERS (NEXT LINE)             *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_NEXT_LINE





      CALL CRT_CMD(CRLF)





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     PRINT A CHARACTER STRING VERBATIM TO THE CRT (NO FORMATTING)     *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_PRINT(OUTPUT_STRING)





      CALL CRT_CMD(OUTPUT_STRING)





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     WRITE A CHARACTER STRING RECORD TO THE TERMINAL AND APPEND CRLF  *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_WRITE(OUTPUT_STRING)





      CALL CRT_CMD(OUTPUT_STRING)
      CALL CRT_CMD(CRLF)





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     DISPLAY A CHARACTER STRING IN DOUBLE-HEIGHT DOUBLE WIDTH FORMAT  *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_LARGE(INPUT_STRING)





      STRING=INPUT_STRING

      IF (VT100) THEN

           CALL CRT_CMD(CR//
     1          VT1_CLEAR_LINE//
     2          VT1_DOUBLE_TOP//STRING(ONE:(LAST_COLUMN/TWO))//
     3          CR//VT1_INDEX_DOWN//
     4          VT1_CLEAR_LINE//
     5          VT1_DOUBLE_BOT//STRING(ONE:(LAST_COLUMN/TWO)))

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     INPUT A CHARACTER STRING VERBATIM FROM THE CRT                   *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT ALLOWS A USER TO INPUT A CHARACTER STRING        *
C     VERBATIM FROM THE CRT, INCLUDING ANY RECORD DELIMITERS           *
C     CHARACTERS WHICH MAY HAVE TERMINATED THE INPUT LINE.             *
C----------------------------------------------------------------------*
      ENTRY CRT_INPUT(INPUT_STRING)





      INPUT_STRING=BLANK

      STATUS=SYS$QIOW(,%VAL(CRT_READER_CHAN),
     1     %VAL(CRT_QIO_READ .OR. %LOC(IO$M_TRMNOECHO)),
     2     IO_STATUS,,,
     3     %REF(INPUT_STRING),%VAL(LEN(INPUT_STRING)),,,,)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     IMAGE READ A CHARACTER STRING FROM THE CRT                       *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT IS FOR THOSE USERS IMPLEMENTING RECORD I/O       *
C     FUNCTIONS ASSOCIATED WITH THE VT100.  NOTE THAT THE RETURNED     *
C     CHARACTER STRING DOES NOT CONTAIN THE RECORD DELIMETER.          *
C----------------------------------------------------------------------*
      ENTRY CRT_READ(INPUT_STRING)






      STATUS=SYS$QIOW(,%VAL(CRT_READER_CHAN),
     1     %VAL(CRT_QIO_READ),
     2     IO_STATUS,,,
     3     %REF(INPUT_STRING),%VAL(LEN(INPUT_STRING)),,,,)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700
      IF (INDEX(INPUT_STRING,EOF) .NE. ZERO) STATUS=%LOC(SS$_ENDOFFILE)
      IF (IO_STATUS(TWO) .EQ. LEN(INPUT_STRING)) GO TO 900
      INPUT_STRING((IO_STATUS(TWO)+ONE):)=BLANK





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     POLL THE CRT FOR A SPECIFIED TIME AND RETURN TYPEAHEAD BUFFER    *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT IS FOR THOSE USERS IMPLEMENTING CHARACTER I/O    *
C     FUNCTIONS ASSOCIATED WITH THE VT100.  THE CRT IS POLLED FOR THE  *
C     SPECIFIED NUMBER OF SECONDS AFTER WHICH THE TYPEAHEAD BUFFER IS  *
C     RETURNED TO THE USER IF A VALID TERMINATOR CODE HAS BEEN         *
C     RECEIVED.  OTHERWISE, THE BUFFER IS RETURNED EMPTY.  NOTE THAT   *
C     THE TERMINATOR SEQUENCE IS INCLUDED WITHIN THE RETURNED STRING.  *
C----------------------------------------------------------------------*
      ENTRY CRT_POLL(INPUT_STRING,TIME_PERIOD)





      INPUT_STRING=BLANK

      STATUS=SYS$QIOW(,%VAL(CRT_READER_CHAN),
     1     %VAL(CRT_QIO_READ),
     2     IO_STATUS,,,
     3     %REF(INPUT_STRING),%VAL(ONE),,,,)
      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     GRAPHIC SYMBOL COMMANDS                                          *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     PLOT A GIVEN POINT USING GRAPHICS MODE HORIZONTAL SCAN LINES 1-9 *
C                                                                      *
C----------------------------------------------------------------------*
C     NOTE THAT THE ORIGIN (1,1) OF THE PLOT IS THE LOWER-LEFT CORNER  *
C     OF THE SCREEN.  PLOTTING SURFACE IS (80,120) IN 80-COLUMN MODE   *
C     AND (132,120) IN 132-COLUMN MODE.  IT IS ASSUMED THAT THE        *
C     ADVANCED VIDEO OPTION HAS BEEN INSTALLED, INSURING THAT THE      *
C     NUMBER OF LINES FOR 80 AND 132 COLUMN MODES ARE EQUIVALENT.      *
C----------------------------------------------------------------------*
      ENTRY CRT_PLOT(X,Y)





      IF (VT100) THEN

           X_COL=JMIN0(JIABS(X),LAST_COLUMN)
           Y_POINT=JMIN0(JIABS(Y),(VT1_ROW_MAX*VT1_SCAN_LINES))

           Y_ROW=(Y_POINT-ONE)/VT1_SCAN_LINES
           YSCAN=Y_POINT-(Y_ROW*VT1_SCAN_LINES)
           Y_ROW=VT1_ROW_MAX-Y_ROW

           IF (YSCAN .EQ. 1) THEN
                POINT=VT1_SYM_SCAN9
           ELSE IF (YSCAN .EQ. 2) THEN
                POINT=VT1_SYM_SCAN7
           ELSE IF (YSCAN .EQ. 3) THEN
                POINT=VT1_SYM_SCAN5
           ELSE IF (YSCAN .EQ. 4) THEN
                POINT=VT1_SYM_SCAN3
           ELSE IF (YSCAN .EQ. 5) THEN
                POINT=VT1_SYM_SCAN1
           ELSE
                POINT=VT1_SYM_DOT
           ENDIF

           CALL INTFMT(Y_ROW,PARM1,LIMIT_ONE)
           CALL INTFMT(X_COL,PARM2,LIMIT_TWO)

           CALL CRT_CMD(ANS//PARM1(ONE:LIMIT_ONE)//
     1          DLM//PARM2(ONE:LIMIT_TWO)//VT1_MOVE//POINT)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           X_POINT=JMIN0(JMAX0(X,ISC_PLOT_X_ORG),ISC_PLOT_X_MAX)
           Y_POINT=JMIN0(JMAX0(Y,ISC_PLOT_Y_ORG),ISC_PLOT_Y_MAX)

           CALL CRT_CMD(
     1          ISC_PLOT_XPOINT//CHAR(X_POINT)//
     2          ISC_PLOT_YPOINT//CHAR(Y_POINT))

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     PLOT A HORIZONTAL (X-AXIS) BAR GRAPH                             *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT ENABLES THE USER TO QUICKLY PRODUCE X-AXIS       *
C     (HORIZONTAL) BAR GRAPHS.  THE PASSED PARAMETERS INCLUDE:         *
C                                                                      *
C          1.  THE STARTING X-COORDINATE OF THE BAR GRAPH              *
C          2.  THE STARTING Y-COORDINATE OF THE BAR GRAPH              *
C          3.  THE MAXIMUM X-COORDINATE OF THE BAR GRAPH               *
C----------------------------------------------------------------------*
      ENTRY CRT_PLOT_X_BAR(X_START,Y_START,X_LIMIT)





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           X_ORG=JMIN0(JMAX0(X_START,ISC_PLOT_X_ORG),ISC_PLOT_X_MAX)
           Y_ORG=JMIN0(JMAX0(Y_START,ISC_PLOT_Y_ORG),ISC_PLOT_Y_MAX)
           X_MAX=JMIN0(JMAX0(X_LIMIT,ISC_PLOT_X_ORG),ISC_PLOT_X_MAX)

           CALL CRT_CMD(
     1          ISC_PLOT_XBAR_X//CHAR(X_ORG)//
     2          ISC_PLOT_XBAR_Y//CHAR(Y_ORG)//
     3          ISC_PLOT_XBAR_M//CHAR(X_MAX))

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     PLOT A VERTICAL (Y-AXIS) BAR GRAPH                               *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT ENABLES THE USER TO QUICKLY PRODUCE Y-AXIS       *
C     (VERTICAL) BAR GRAPHS.  THE PASSED PARAMETERS INCLUDE:           *
C                                                                      *
C          1.  THE STARTING X-COORDINATE OF THE BAR GRAPH              *
C          2.  THE STARTING Y-COORDINATE OF THE BAR GRAPH              *
C          3.  THE MAXIMUM Y-COORDINATE OF THE BAR GRAPH               *
C----------------------------------------------------------------------*
      ENTRY CRT_PLOT_Y_BAR(X_START,Y_START,Y_LIMIT)





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           X_ORG=JMIN0(JMAX0(X_START,ISC_PLOT_X_ORG),ISC_PLOT_X_MAX)
           Y_ORG=JMIN0(JMAX0(Y_START,ISC_PLOT_Y_ORG),ISC_PLOT_Y_MAX)
           Y_MAX=JMIN0(JMAX0(Y_LIMIT,ISC_PLOT_Y_ORG),ISC_PLOT_Y_MAX)

           CALL CRT_CMD(
     1          ISC_PLOT_YBAR_Y//CHAR(Y_ORG)//
     2          ISC_PLOT_YBAR_X//CHAR(X_ORG)//
     3          ISC_PLOT_YBAR_M//CHAR(Y_MAX))

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     PLOT A VECTOR FROM THE LAST POINT (X0,Y0) TO A NEW POINT (X1,Y1) *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT ENABLES THE USER TO QUICKLY PRODUCE A VECTOR     *
C     (LINE) TO A NEW POINT.  THE PASSED PARAMETERS INCLUDE:           *
C                                                                      *
C          1.  THE ENDING X-COORDINATE OF THE VECTOR                   *
C          2.  THE ENDING Y-COORDINATE OF THE VECTOR                   *
C----------------------------------------------------------------------*
      ENTRY CRT_PLOT_VECTOR(X,Y)





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           X_MAX=JMIN0(JMAX0(X,ISC_PLOT_X_ORG),ISC_PLOT_X_MAX)
           Y_MAX=JMIN0(JMAX0(Y,ISC_PLOT_Y_ORG),ISC_PLOT_Y_MAX)

           CALL CRT_CMD(
     1          ISC_PLOT_LINE_X//CHAR(X_MAX)//
     2          ISC_PLOT_LINE_Y//CHAR(Y_MAX))

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     PLOT A LINE FROM (X0,Y0) TO (X1,Y1)                              *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT ENABLES THE USER TO QUICKLY PRODUCE A LINE       *
C     (VECTOR) BETWEEN TWO POINTS.  THE PASSED PARAMETERS INCLUDE:     *
C                                                                      *
C          1.  THE STARTING X-COORDINATE OF THE VECTOR TO BE DRAWN     *
C          2.  THE STARTING Y-COORDINATE OF THE VECTOR TO BE DRAWN     *
C          3.  THE MAXIMUM X-COORDINATE OF THE VECTOR TO BE DRAWN      *
C          4.  THE MAXIMUM Y-COORDINATE OF THE VECTOR TO BE DRAWN      *
C----------------------------------------------------------------------*
      ENTRY CRT_PLOT_LINE(X_START,Y_START,X_STOP,Y_STOP)





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           X_ORG=JMIN0(JMAX0(X_START,ISC_PLOT_X_ORG),ISC_PLOT_X_MAX)
           Y_ORG=JMIN0(JMAX0(Y_START,ISC_PLOT_Y_ORG),ISC_PLOT_Y_MAX)
           X_MAX=JMIN0(JMAX0(X_STOP,ISC_PLOT_X_ORG),ISC_PLOT_X_MAX)
           Y_MAX=JMIN0(JMAX0(Y_STOP,ISC_PLOT_Y_ORG),ISC_PLOT_Y_MAX)

           CALL CRT_CMD(
     1          ISC_PLOT_XPOINT//CHAR(X_ORG)//
     2          ISC_PLOT_YPOINT//CHAR(Y_ORG)//
     3          ISC_PLOT_LINE_X//CHAR(X_MAX)//
     4          ISC_PLOT_LINE_Y//CHAR(Y_MAX))

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     GRAPHIC CHARACTER SET (LINE DRAWING) COMMANDS                    *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     BLANK                                                            *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_BLANK





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_BLANK)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     DIAMOND                                                          *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_DIAMOND





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_DIAMOND)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CHECK BOARD (ERROR INDICATOR)                                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_CHECK





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_CHECK)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     HT (HORIZONTAL TAB)                                              *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_HT





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_HT)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     FF (FORM FEED)                                                   *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_FF





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_FF)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CR (CARRIAGE RETURN)                                             *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_CR





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_CR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     LF (LINE FEED)                                                   *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_LF





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_LF)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     DEGREE SYMBOL                                                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_DEGREE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_DEGREE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     PLUS/MINUS +/-                                                   *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_PL_MIN





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_PL_MIN)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     NL (NEW LINE)                                                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_NL





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_NL)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     VT (VERTICAL TAB)                                                *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_VT





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_VT)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     LOWER-RIGHT CORNER                                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_LRC





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_LRC)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     UPPER-RIGHT CORNER                                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_URC





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_URC)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     UPPER-LEFT CORNER                                                *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_ULC





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_ULC)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     LOWER-LEFT CORNER                                                *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_LLC





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_LLC)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CROSSING LINES                                                   *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_CROSS





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_CROSS)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     HORIZONTAL LINE - SCAN 1                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_SCAN1





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_SCAN1)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     HORIZONTAL LINE - SCAN 3                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_SCAN3





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_SCAN3)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     HORIZONTAL LINE - SCAN 5                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_SCAN5





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_SCAN5)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     HORIZONTAL LINE - SCAN 7                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_SCAN7





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_SCAN7)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     HORIZONTAL LINE - SCAN 9                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_SCAN9





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_SCAN9)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     LEFT "T"                                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_LEFT_T





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_LEFT_T)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     RIGHT "T"                                                        *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_RIGHT_T





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_RIGHT_T)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     BOTTOM "T"                                                       *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_LOWER_T





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_LOWER_T)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     TOP "T"                                                          *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_UPPER_T





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_UPPER_T)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     VERTICAL BAR                                                     *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_V_BAR





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_V_BAR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     VERTICAL LINE                                                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_V_LINE(COUNT)





      IF (VT100) THEN

           REPEAT_COUNT=COUNT
           CALL CRT_CMD(VT1_SYM_V_BAR//
     1          VT1_MOVE_DOWN1//BACK_SPACE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     LESS THAN OR EQUAL TO (.LE.)                                     *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_LE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_LE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     GREATER THAN OR EQUAL TO (.GE.)                                  *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_GE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_GE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     PI (MATHEMATICAL)                                                *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_PI





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_PI)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     NOT EQUAL TO (.NE.)                                              *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_NE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_NE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     UK POUND SIGN                                                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_POUND





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_POUND)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CENTERED DOT                                                     *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SYM_DOT





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SYM_DOT)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     HORIZONTAL AND VERTICAL MARGIN COMMANDS                          *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SET TERMINAL PHYSICAL FORM LENGTH                                *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_FORM_LENGTH(PITCH)





      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL INTFMT(PITCH,PARM1,LIMIT)
           CALL CRT_CMD(ANS//PARM1(ONE:LIMIT)//LA1_FORM_LENGTH)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET HORIZONTAL MARGIN COMMAND                                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_MARGIN(FIRST,LAST)






      IF (VT100) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL INTFMT(FIRST,PARM1,LIMIT_ONE)
           CALL INTFMT(LAST,PARM2,LIMIT_TWO)
           CALL CRT_CMD(ANS//PARM1(ONE:LIMIT_ONE)//
     1          DLM//PARM2(ONE:LIMIT_TWO)//LA1_MARGIN)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF

C----------------------------------------------------------------------*
C                                                                      *
C     SET VERTICAL MARGIN (SCROLLING REGION) COMMAND                   *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_REGION(FIRST,LAST)





      IF (VT100) THEN

           CALL INTFMT(FIRST,PARM1,LIMIT_ONE)
           CALL INTFMT(LAST,PARM2,LIMIT_TWO)
           CALL CRT_CMD(ANS//PARM1(ONE:LIMIT_ONE)//
     1          DLM//PARM2(ONE:LIMIT_TWO)//VT1_SET_REGION)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL INTFMT(FIRST,PARM1,LIMIT_ONE)
           CALL INTFMT(LAST,PARM2,LIMIT_TWO)
           CALL CRT_CMD(ANS//PARM1(ONE:LIMIT_ONE)//
     1          DLM//PARM2(ONE:LIMIT_TWO)//LA1_REGION)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     TAB STOP COMMANDS                                                *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SET TAB AT CURRENT COLUMN                                        *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SET_TAB





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SET_TAB)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CLEAR TAB AT CURRENT COLUMN                                      *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_CLEAR_TAB





      IF (VT100) THEN

           CALL CRT_CMD(VT1_CLEAR_TAB)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     CLEAR ALL TABS                                                   *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_CLEAR_TABS





      IF (VT100) THEN

           CALL CRT_CMD(VT1_CLEAR_TABS)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     MODE CONVERSION COMMANDS                                         *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     SET NEW LINE MODE                                                *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_NL_MODE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_NL_MODE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_NL_MODE)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET LINE FEED MODE                                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_LF_MODE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_LF_MODE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_LF_MODE)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET APPLICATION MODE FOR CURSOR KEY                              *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_CURSOR_APP





      IF (VT100) THEN

           CALL CRT_CMD(VT1_CURSOR_APP)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET CURSOR MODE FOR CURSOR KEY                                   *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_CURSOR_KEY





      IF (VT100) THEN

           CALL CRT_CMD(VT1_CURSOR_KEY)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET ANSI (VT100) MODE                                            *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_ANSI
      ENTRY CRT_VT100





      DO POINTER=ONE,MAXCRT
           DEVICE_TABLE(POINTER)=.FALSE.
      ENDDO

      VT100=.TRUE.
      CALL CRT_CMD(VT1_ANSI)





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET VT52 MODE                                                    *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_VT52





      DO POINTER=ONE,MAXCRT
           DEVICE_TABLE(POINTER)=.FALSE.
      ENDDO

      VT52=.TRUE.
      CALL CRT_CMD(VT1_VT52)





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET INTECOLOR 8001 (COLOR GRAPHICS TERMINAL) MODE                *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_ISC_8001





      DO POINTER=ONE,MAXCRT
           DEVICE_TABLE(POINTER)=.FALSE.
      ENDDO

      ISC_8001=.TRUE.





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET 132 COLUMN MODE                                              *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_132_COLUMN





      IF (VT100) THEN

           LAST_COLUMN=VT1_WIDE_WIDTH
           CALL CRT_CMD(VT1_132_COLUMN)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET 80 COLUMN MODE                                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_80_COLUMN





      IF (VT100) THEN

           LAST_COLUMN=VT1_COLUMN_MAX
           CALL CRT_CMD(VT1_80_COLUMN)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET SMOOTH SCROLLING MODE                                        *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_SMOOTH





      IF (VT100) THEN

           CALL CRT_CMD(VT1_SMOOTH)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET JUMP SCROLLING MODE                                          *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_JUMP





      IF (VT100) THEN

           CALL CRT_CMD(VT1_JUMP)       ! VT100 FIRMWARE BUG:
           CALL DELAY(100)              ! MUST DELAY FOR JUMP SCROLL

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET REVERSE SCREEN MODE                                          *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_PAGE_LIGHT





      IF (VT100) THEN

           CALL CRT_CMD(VT1_PAGE_LIGHT)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET NORMAL SCREEN MODE                                           *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_PAGE_DARK





      IF (VT100) THEN

           CALL CRT_CMD(VT1_PAGE_DARK)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET RELATIVE ORIGIN MODE                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_RELATIVE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_RELATIVE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET ABSOLUTE ORIGIN MODE                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_ABSOLUTE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_ABSOLUTE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET WRAPAROUND MODE ON                                           *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_WRAP_ON





      IF (VT100) THEN

           CALL CRT_CMD(VT1_WRAP_ON)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET WRAPAROUND MODE OFF                                          *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_WRAP_OFF





      IF (VT100) THEN

           CALL CRT_CMD(VT1_WRAP_OFF)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET AUTO REPEAT MODE ON                                          *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_REPEAT_ON





      IF (VT100) THEN

           CALL CRT_CMD(VT1_REPEAT_ON)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET AUTO REPEAT MODE OFF                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_REPEAT_OFF





      IF (VT100) THEN

           CALL CRT_CMD(VT1_REPEAT_OFF)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET INTERLACE MODE ON                                            *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_RETRACE_ON





      IF (VT100) THEN

           CALL CRT_CMD(VT1_RETRACE_ON)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET INTERLACE MODE OFF                                           *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_RETRACE_OFF





      IF (VT100) THEN

           CALL CRT_CMD(VT1_RETRACE_OFF)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET GRAPHICS PROCESSOR OPTION ON                                 *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_GRAPHIC_ON





      IF (VT100) THEN

           CALL CRT_CMD(VT1_GRAPHIC_ON)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET GRAPHICS PROCESSOR OPTION OFF                                *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_GRAPHIC_OFF





      IF (VT100) THEN

           CALL CRT_CMD(VT1_GRAPHIC_OFF)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET KEYPAD APPLICATION MODE                                      *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_KEYPAD_APP





      IF (VT100) THEN

           CALL CRT_CMD(VT1_KEYPAD_APP)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(VT5_KEYPAD_APP)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_KEYPAD_APP)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SET KEYPAD NUMERIC MODE                                          *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_KEYPAD_NUM





      IF (VT100) THEN

           CALL CRT_CMD(VT1_KEYPAD_NUM)

      ELSE IF (VT52) THEN

           CALL CRT_CMD(VT5_KEYPAD_NUM)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_KEYPAD_NUM)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     VT100 REPORT AND RESET COMMANDS                                  *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     CURSOR POSITION REPORT                                           *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT PROMPTS THE USER TERMINAL FOR THE CURRENT CURSOR *
C     POSITION AND RETURNS THIS VALUE TO THE USER VIA THE ROW AND COL  *
C     ARGUMENTS.  IF THE ROW AND COLUMN ARE UNKNOWN, THE ARGUMENTS ARE *
C     RETURNED AS ZEROS.                                               *
C----------------------------------------------------------------------*
      ENTRY CRT_POSITION(ROW,COL)





      ROW=ZERO
      COL=ZERO

      IF (VT100) THEN

           CALL CRT_CMD(VT1_POSITION)
           STRING=BLANK
           STATUS=SYS$QIOW(,%VAL(CRT_READER_CHAN),
     1          %VAL(CRT_QIO_READ .OR. %LOC(IO$M_TRMNOECHO)),
     2          IO_STATUS,,,
     3          %REF(STRING),%VAL(LEN(STRING)),,,,)
           IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700
           LIMIT_ONE=INDEX(STRING,ANS)
           IF (LIMIT_ONE .EQ. ZERO) GO TO 900
           LIMIT_TWO=INDEX(STRING,DLM)
           IF (LIMIT_TWO .EQ. ZERO) GO TO 900
           PARM1=STRING((LIMIT_ONE+TWO):(LIMIT_TWO-ONE))
           LIMIT_ONE=INDEX(STRING,VT1_POS_REPORT)
           PARM2=STRING((LIMIT_TWO+ONE):(LIMIT_ONE-ONE))
           CALL DECODE_INTEGER(PARM1,ROW,NODATA,ERROR)
           CALL DECODE_INTEGER(PARM2,COL,NODATA,ERROR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     DEVICE STATUS REPORT                                             *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_STATUS





      IF (VT100) THEN

           CALL CRT_CMD(VT1_STATUS)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     DEVICE ATTRIBUTES REPORT                                         *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_ATTRIBUTES





      IF (VT100) THEN

           CALL CRT_CMD(VT1_ATTRIBUTES)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     REQUEST TERMINAL PARAMETERS REPORT                               *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_REQUEST(REQUEST_PARAM)





      IF (VT100) THEN

           CALL INTFMT(REQUEST_PARAM,PARM1,LIMIT_ONE)
           CALL CRT_CMD(ANS//PARM1(ONE:LIMIT_ONE)//VT1_REQUEST)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     AUTO-ANSWERBACK MESSAGE COMMAND                                  *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_ANSWERBACK





      IF (VT100) THEN

           CALL CRT_CMD(VT1_ANSWERBACK)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           CALL CRT_CMD(LA1_ANSWERBACK)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     VT100 RESET                                                      *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_RESET





      IF (VT100) THEN

           CALL CRT_CMD(VT1_RESET)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     VT100 SCREEN ALIGNMENT TEST                                      *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_ALIGNMENT





      IF (VT100) THEN

           CALL CRT_CMD(VT1_ALIGNMENT)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     VT100 CONFIDENCE TESTS                                           *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_TEST(TEST)





      IF (VT100) THEN

           CALL INTFMT(TEST,PARM1,LIMIT_ONE)
           CALL CRT_CMD(VT1_TEST//PARM1(ONE:LIMIT_ONE)//VT1_TEST_ENDFIX)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     VT100 AUXILIARY I/O PORT COMMANDS                                *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     PRINT ENTIRE VT100 SCREEN TO AUXILIARY I/O PORT                  *
C                                                                      *
C----------------------------------------------------------------------*
      ENTRY CRT_PRINT_PAGE





      IF (VT100) THEN

           CALL CRT_CMD(VT1_PRINT_PAGE)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     EXIT PRINTER CONTROLLER MODE (TURN I/O PORT OFF)                 *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT DISABLES THE AUXILIARY PRINTER PORT INTERFACE    *
C     AND RETURNS CONTROL AND DISPLAY FUNCTIONS TO THE MAIN DEVICE     *
C     INTERFACE AND ITS ASSOCIATED DEVICE INTERFACE DRIVER.            *
C----------------------------------------------------------------------*
      ENTRY CRT_PRINTER_OFF





C----------------------------------------------------------------------*
C     SWITCH CONTEXTS FOR CURRENT DEVICE TABLE                         *
C----------------------------------------------------------------------*
      DO POINTER=ONE,MAXCRT
           FLAG=DEVICE_AUXIO(POINTER)
           DEVICE_AUXIO(POINTER)=DEVICE_TABLE(POINTER)
           DEVICE_TABLE(POINTER)=FLAG
      ENDDO
C----------------------------------------------------------------------*
C     NOW PHYSICALLY SWITCH CONTEXTS FOR THE CURRENT HARDWARE DEVICE   *
C----------------------------------------------------------------------*
      IF (VT100) THEN

           CALL CRT_CMD(VT1_PRINTER_OFF)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     ENTER PRINTER CONTROLLER MODE (TURN I/O PORT ON)                 *
C                                                                      *
C----------------------------------------------------------------------*
C     THIS ENTRYPOINT ENABLES THE AUXILIARY I/O PORT ON THE CURRENT    *
C     TERMINAL DEVICE IF THE DEVICE SUPPORTS THIS FEATURE.  AFTER      *
C     EXECUTION OF THIS ENTRYPOINT, ALL DATA DIRECTED TO THE DEVICE    *
C     WILL BE ROUTED TO THE AUXILIARY PRINTER PORT AND WILL BE         *
C     INTERPRETED BY THE CORRESPONDING DEVICE INTERFACE DRIVER.        *
C----------------------------------------------------------------------*
      ENTRY CRT_PRINTER_ON





C----------------------------------------------------------------------*
C     NOW PHYSICALLY SWITCH CONTEXTS FOR THE CURRENT HARDWARE DEVICE   *
C----------------------------------------------------------------------*
      IF (VT100) THEN

           CALL CRT_CMD(VT1_PRINTER_ON)

      ELSE IF (VT52) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HAZELTINE) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (HP264X) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (ISC_8001) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA36) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (LA120) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (PRINTRONIX) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (QUME) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ELSE IF (UNKNOWN) THEN

           STATUS=%LOC(SS$_DEVCMDERR)

      ENDIF
C----------------------------------------------------------------------*
C     SWITCH CONTEXTS FOR CURRENT DEVICE TABLE                         *
C----------------------------------------------------------------------*
      DO POINTER=ONE,MAXCRT
           FLAG=DEVICE_AUXIO(POINTER)
           DEVICE_AUXIO(POINTER)=DEVICE_TABLE(POINTER)
           DEVICE_TABLE(POINTER)=FLAG
      ENDDO





      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SECTION 700                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     ERROR PROCESSING                                                 *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C     DISPLAY THE VAX/VMS ERROR STATUS TO THE OUTPUT DEVICE            *
C----------------------------------------------------------------------*
  700 CONTINUE

      CALL SYS$GETMSG(%VAL(STATUS),LIMIT,STRING,%VAL(FULL),)
      CALL LIB$PUT_OUTPUT(STRING(ONE:LIMIT))

      GO TO 900

C----------------------------------------------------------------------*
C                                                                      *
C     SECTION 800                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     SET TERMINAL MODE CHARACTERISTICS                                *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
  800 CONTINUE
      STATUS=SYS$QIOW(,%VAL(CRT_READER_CHAN),
     1     %VAL(%LOC(IO$_SETMODE)),
     2     IO_STATUS,,,
     3     QIO_MODE_BUF,,,,,)

      IF (STATUS .NE. %LOC(SS$_NORMAL)) GO TO 700

C----------------------------------------------------------------------*
C                                                                      *
C     SECTION 900                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C     SUBROUTINE TERMINATION                                           *
C                                                                      *
C                                                                      *
C                                                                      *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     RECORD STATUS OF LAST I/O OPERATION                              *
C                                                                      *
C----------------------------------------------------------------------*
  900 CONTINUE
      CRT_DRIVER=STATUS
C----------------------------------------------------------------------*
C                                                                      *
C     END OF SUBROUTINE VT100_DRIVER                                   *
C                                                                      *
C----------------------------------------------------------------------*
      RETURN
      END

      SUBROUTINE CRT_CMD(STRING)
C----------------------------------------------------------------------*
C                                                                      *
C     SUBROUTINE:   CRT_CMD                                            *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     LANGUAGE:     VAX-11 FORTRAN IV V2.0-2                           *
C                                                                      *
C                                                                      *
C     SYSTEM:       VAX-11/780                                         *
C                                                                      *
C                                                                      *
C                   MOSTEK CORPORATION                                 *
C                                                                      *
C                                                                      *
C                   COMPUTER AIDED DESIGN DIVISION                     *
C                                                                      *
C                                                                      *
C                   1215 WEST CROSBY ROAD                              *
C                                                                      *
C                                                                      *
C                   CARROLLTON, TEXAS  75006                           *
C                                                                      *
C                                                                      *
C                   (214)  323-8813                                    *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     PROGRAMMER:   KEVIN KLUGHART                                     *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     DATE:         02-08-81 @ 19:00 CDST                              *
C                                                                      *
C----------------------------------------------------------------------*
C                                                                      *
C     PURPOSE:      THIS SUBROUTINE IS DESIGNED AS THE MAIN CRT        *
C                                                                      *
C                                                                      *
C                   COMMAND OUTPUT DISPLAY SUBROUTINE.  THE SELECTED   *
C                                                                      *
C                                                                      *
C                   CRT COMMAND IS PASSED BY STRING DESCRIPTOR TO      *
C                                                                      *
C                                                                      *
C                   THIS SUBROUTINE WHICH IN TURN OUTPUTS IT TO THE    *
C                                                                      *
C                                                                      *
C                   VIDEO TERMINAL.                                    *
C                                                                      *
C----------------------------------------------------------------------*

C----------------------------------------------------------------------*
C                                                                      *
C     VAX/VMS QIO DATA STRUCTURES                                      *
C                                                                      *
C----------------------------------------------------------------------*
C     THE GENERAL FORMAT FOR THE SYS$QIO AND SYS$QIOW SYSTEM SERVICE   *
C     CALLS IS AS FOLLOWS:                                             *
C                                                                      *
C     SYS$QIO[W] [EFN],CHAN,                                           *
C                FUNC,                                                 *
C                [IOSB],[ASTADR],[ASTPRM],                             *
C                [P1],[P2],[P3],[P4],[P5],[P6]                         *
C----------------------------------------------------------------------*

C----------------------------------------------------------------------*
C     PARAMETERS                                                       *
C----------------------------------------------------------------------*
      PARAMETER FOUR             = 4              ! USEFUL CONSTANT
      PARAMETER ONE              = 1              ! USEFUL CONSTANT
C----------------------------------------------------------------------*
C     CHARACTER                                                        *
C----------------------------------------------------------------------*
      CHARACTER*(*) STRING
C----------------------------------------------------------------------*
C     INTEGER*2                                                        *
C----------------------------------------------------------------------*
      INTEGER*2 IO_STATUS      (FOUR)
C----------------------------------------------------------------------*
C     INTEGER*4                                                        *
C----------------------------------------------------------------------*
      INTEGER*4 COUNTER
      INTEGER*4 CRT_QIO_READ
      INTEGER*4 CRT_QIO_WRITE
      INTEGER*4 CRT_READER_CHAN
      INTEGER*4 CRT_WRITER_CHAN
      INTEGER*4 REPEAT_COUNT
      INTEGER*4 STATUS

      INCLUDE 'CRT.CMM'

C----------------------------------------------------------------------*
C     SECTION 100                                                      *
C----------------------------------------------------------------------*
C     TRANSFER CRT COMMAND TO USER VIDEO TERMINAL                      *
C----------------------------------------------------------------------*
  100 CONTINUE
      
      DO COUNTER=ONE,REPEAT_COUNT
           STATUS=SYS$QIOW(,%VAL(CRT_WRITER_CHAN),
     1          %VAL(CRT_QIO_WRITE),
     2          IO_STATUS,,,
     3          %REF(STRING),%VAL(LEN(STRING)),,,,)
      ENDDO

      REPEAT_COUNT=ONE
C----------------------------------------------------------------------*
C     END OF SUBROUTINE CRT_CMD                                        *
C----------------------------------------------------------------------*
      RETURN
      END
